@@ -677,8 +677,8 @@ static const char vertex_shader_gl[] =
 "\n"
 "    // Multiply the diffuse value by the vertex color (which is fixed in this case)\n"
 "    // to get the actual color that we will use to draw this vertex with\n"
-"    float diffuse = max(dot(N, L), 0.0);\n"
-"    Color = diffuse * MaterialColor;\n"
+"    float diffuse = (dot(N, L) + 1.0) * 0.5;\n"
+"    Color = vec4(diffuse * MaterialColor.rgb, 1.0);\n"
 "\n"
 "    // Transform the position to clip coordinates\n"
 "    gl_Position = ModelViewProjectionMatrix * vec4(position, 1.0);\n"
@@ -715,8 +715,8 @@ static const char vertex_shader_gles[] =
 "\n"
 "    // Multiply the diffuse value by the vertex color (which is fixed in this case)\n"
 "    // to get the actual color that we will use to draw this vertex with\n"
-"    float diffuse = max(dot(N, L), 0.0);\n"
-"    Color = diffuse * MaterialColor;\n"
+"    float diffuse = (dot(N, L) + 1.0) * 0.5;\n"
+"    Color = vec4(diffuse * MaterialColor.rgb, 1.0);\n"
 "\n"
 "    // Transform the position to clip coordinates\n"
 "    gl_Position = ModelViewProjectionMatrix * vec4(position, 1.0);\n"
