@@ -105,14 +105,16 @@ set_missing_attribute_error (GError     **error,
                "Missing attribute: %s", name);
 }
 
-static void
+static gboolean
 markup_filter_attributes (const char *element_name,
                           const char **attribute_names,
                           const char **attribute_values,
+                          GError     **error,
                           const char  *name,
                           ...)
 {
   va_list ap;
+  guint64 filtered = 0;
 
   va_start (ap, name);
   while (name)
@@ -121,12 +123,37 @@ markup_filter_attributes (const char *element_name,
 
       ptr = va_arg (ap, const char **);
 
-      *ptr = NULL;
+      if (ptr)
+        *ptr = NULL;
+
+      /* Ignoring a whole namespace */
+      if (g_str_has_suffix (name, ":*"))
+        {
+          GPatternSpec *spec = g_pattern_spec_new (name);
+
+          for (int i = 0; attribute_names[i]; i++)
+            {
+              if (filtered & (1ull << i))
+                continue;
+
+              if (g_pattern_spec_match_string (spec, attribute_names[i]))
+                {
+                  filtered |= 1ull << i;
+                  continue;
+                }
+            }
+        }
+
       for (int i = 0; attribute_names[i]; i++)
         {
+          if (filtered & (1ull << i))
+            continue;
+
           if (strcmp (attribute_names[i], name) == 0)
             {
-              *ptr = attribute_values[i];
+              if (ptr)
+                *ptr = attribute_values[i];
+              filtered |= 1ull << i;
               break;
             }
         }
@@ -135,6 +162,22 @@ markup_filter_attributes (const char *element_name,
     }
 
   va_end (ap);
+
+  for (guint i = 0; attribute_names[i]; i++)
+    {
+      if ((filtered & (1ull << i)) == 0)
+        {
+          g_set_error (error, G_MARKUP_ERROR,
+                       G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
+                       "attribute '%s' is invalid for element '%s'",
+                       attribute_names[i], element_name);
+          g_print ("unhandled attribute %s\n", attribute_names[i]);
+          return FALSE;
+        }
+    }
+
+  return TRUE;
+
 }
 
 static GskPath *
@@ -212,6 +255,7 @@ start_element_cb (GMarkupParseContext  *context,
       markup_filter_attributes (element_name,
                                 attribute_names,
                                 attribute_values,
+                                NULL,
                                 "width", &width_attr,
                                 "height", &height_attr,
                                 NULL);
@@ -265,6 +309,7 @@ start_element_cb (GMarkupParseContext  *context,
       markup_filter_attributes (element_name,
                                 attribute_names,
                                 attribute_values,
+                                NULL,
                                 "cx", &cx_attr,
                                 "cy", &cy_attr,
                                 "r", &r_attr,
@@ -323,6 +368,7 @@ start_element_cb (GMarkupParseContext  *context,
       markup_filter_attributes (element_name,
                                 attribute_names,
                                 attribute_values,
+                                NULL,
                                 "x", &x_attr,
                                 "y", &y_attr,
                                 "width", &width_attr,
@@ -400,6 +446,7 @@ start_element_cb (GMarkupParseContext  *context,
       markup_filter_attributes (element_name,
                                 attribute_names,
                                 attribute_values,
+                                NULL,
                                 "d", &path_attr,
                                 NULL);
 
@@ -425,32 +472,33 @@ start_element_cb (GMarkupParseContext  *context,
 
   g_assert (path != NULL);
 
-  if (!g_markup_collect_attributes (element_name,
-                                    attribute_names,
-                                    attribute_values,
-                                    error,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "class", &class_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "opacity", &opacity_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "fill", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "fill-rule", &fill_rule_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "fill-opacity", &fill_opacity_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-width", &stroke_width_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-opacity", &stroke_opacity_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-linecap", &stroke_linecap_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-linejoin", &stroke_linejoin_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-miterlimit", &stroke_miterlimit_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-dasharray", &stroke_dasharray_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-dashoffset", &stroke_dashoffset_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "style", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "id", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "color", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "overflow", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "d", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "cx", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "cy", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "r", NULL,
-                                    G_MARKUP_COLLECT_INVALID))
+  if (!markup_filter_attributes (element_name,
+                                 attribute_names,
+                                 attribute_values,
+                                 error,
+                                 "class", &class_attr,
+                                 "opacity", &opacity_attr,
+                                 "fill-rule", &fill_rule_attr,
+                                 "fill-opacity", &fill_opacity_attr,
+                                 "stroke-width", &stroke_width_attr,
+                                 "stroke-opacity", &stroke_opacity_attr,
+                                 "stroke-linecap", &stroke_linecap_attr,
+                                 "stroke-linejoin", &stroke_linejoin_attr,
+                                 "stroke-miterlimit", &stroke_miterlimit_attr,
+                                 "stroke-dasharray", &stroke_dasharray_attr,
+                                 "stroke-dashoffset", &stroke_dashoffset_attr,
+                                 "fill", NULL,
+                                 "stroke", NULL,
+                                 "style", NULL,
+                                 "id", NULL,
+                                 "color", NULL,
+                                 "overflow", NULL,
+                                 "d", NULL,
+                                 "cx", NULL,
+                                 "cy", NULL,
+                                 "r", NULL,
+                                 "gtk:*", NULL,
+                                 NULL))
     return;
 
   fill_opacity = 1;
@@ -566,7 +614,9 @@ start_element_cb (GMarkupParseContext  *context,
   else
     fill_rule = GSK_FILL_RULE_WINDING;
 
-  stroke = gsk_stroke_new (1);
+  stroke = gsk_stroke_new (2);
+  gsk_stroke_set_line_cap (stroke, GSK_LINE_CAP_ROUND);
+  gsk_stroke_set_line_join (stroke, GSK_LINE_JOIN_ROUND);
 
   if (stroke_width_attr)
     {
