@@ -67,6 +67,8 @@ activate_about (GSimpleAction *action,
   char *os_name;
   char *os_version;
   GtkWidget *dialog;
+  GFile *logo_file;
+  GtkIconPaintable *logo;
 
   os_name = g_get_os_info (G_OS_INFO_KEY_NAME);
   os_version = g_get_os_info (G_OS_INFO_KEY_VERSION_ID);
@@ -106,6 +108,8 @@ activate_about (GSimpleAction *action,
                              gtk_get_minor_version (),
                              gtk_get_micro_version ());
 
+  logo_file = g_file_new_for_uri ("resource:///org/gtk/gtk4/node-editor/icons/apps/org.gtk.gtk4.NodeEditor.svg");
+  logo = gtk_icon_paintable_new_for_file (logo_file, 64, 1);
   dialog = g_object_new (GTK_TYPE_ABOUT_DIALOG,
                          "transient-for", gtk_application_get_active_window (app),
                          "program-name", g_strcmp0 (PROFILE, "devel") == 0
@@ -117,10 +121,12 @@ activate_about (GSimpleAction *action,
                          "website", "http://www.gtk.org",
                          "comments", "Program to test GTK rendering",
                          "authors", (const char *[]){ "Benjamin Otte", "Timm Bäder", NULL},
-                         "logo-icon-name", "org.gtk.gtk4.NodeEditor",
+                         "logo", logo,
                          "title", "About GTK Node Editor",
                          "system-information", s->str,
                          NULL);
+  g_object_unref (logo);
+  g_object_unref (logo_file);
 
   gtk_about_dialog_add_credit_section (GTK_ABOUT_DIALOG (dialog),
                                        "Artwork by", (const char *[]) { "Jakub Steiner", NULL });
@@ -260,6 +266,31 @@ node_editor_application_class_init (NodeEditorApplicationClass *class)
   application_class->open = node_editor_application_open;
 }
 
+static int
+local_options (GApplication *app,
+               GVariantDict *options,
+               gpointer      data)
+{
+  gboolean reset = FALSE;
+
+  g_variant_dict_lookup (options, "reset", "b", &reset);
+
+  if (reset)
+    {
+      char *path;
+
+      path = get_autosave_path ("-unsafe");
+      g_remove (path);
+      g_free (path);
+      path = get_autosave_path (NULL);
+      g_remove (path);
+      g_free (path);
+    }
+
+  return -1;
+}
+
+
 NodeEditorApplication *
 node_editor_application_new (void)
 {
@@ -279,5 +310,7 @@ node_editor_application_new (void)
 
   g_application_add_main_option (G_APPLICATION (app), "reset", 0, 0,G_OPTION_ARG_NONE, "Remove autosave content", NULL);
 
+  g_signal_connect (app, "handle-local-options", G_CALLBACK (local_options), NULL);
+
   return app;
 }
