@@ -89,6 +89,22 @@ gdk_cairo_format_to_memory_format (cairo_format_t format)
   }
 }
 
+static inline cairo_format_t
+gdk_cairo_format_for_content (cairo_content_t content)
+{
+  switch (content)
+    {
+      case CAIRO_CONTENT_COLOR:
+        return CAIRO_FORMAT_RGB24;
+      case CAIRO_CONTENT_ALPHA:
+        return CAIRO_FORMAT_A8;
+      case CAIRO_CONTENT_COLOR_ALPHA:
+        return CAIRO_FORMAT_ARGB32;
+      default:
+        g_return_val_if_reached (CAIRO_FORMAT_ARGB32);
+    }
+}
+
 static inline void
 gdk_cairo_set_source_color (cairo_t        *cr,
                             GdkColorState  *ccs,
@@ -163,12 +179,13 @@ gdk_cairo_surface_convert_color_state (cairo_surface_t *surface,
   if (status == CAIRO_STATUS_SUCCESS && width > 0 && height > 0)
     {
       gdk_memory_convert_color_state (cairo_image_surface_get_data (image_surface),
-                                      cairo_image_surface_get_stride (image_surface),
-                                      gdk_cairo_format_to_memory_format (cairo_image_surface_get_format (image_surface)),
+                                      &GDK_MEMORY_LAYOUT_SIMPLE (
+                                          gdk_cairo_format_to_memory_format (cairo_image_surface_get_format (image_surface)),
+                                          width,
+                                          height,
+                                          cairo_image_surface_get_stride (image_surface)),
                                       source,
-                                      target,
-                                      width,
-                                      height);
+                                      target);
     }
 
   cairo_surface_mark_dirty (image_surface);
@@ -177,3 +194,42 @@ gdk_cairo_surface_convert_color_state (cairo_surface_t *surface,
   cairo_surface_mark_dirty (surface);
 }
 
+static inline cairo_region_t *
+gdk_cairo_region_scale_grow (const cairo_region_t *region,
+                             double                scale_x,
+                             double                scale_y)
+{
+  cairo_region_t *result;
+
+  result = cairo_region_create ();
+  for (int i = 0; i < cairo_region_num_rectangles (region); i++)
+    {
+      cairo_rectangle_int_t rect;
+      cairo_region_get_rectangle (region, i, &rect);
+      cairo_region_union_rectangle (result, &(cairo_rectangle_int_t) {
+                                      .x = (int) floor (rect.x * scale_x),
+                                      .y = (int) floor (rect.y * scale_y),
+                                      .width = (int) ceil ((rect.x + rect.width) * scale_x) - floor (rect.x * scale_x),
+                                      .height = (int) ceil ((rect.y + rect.height) * scale_y) - floor (rect.y * scale_y),
+                                    });
+    }
+
+  return result;
+}
+
+static inline char *
+gdk_cairo_region_to_debug_string (const cairo_region_t *region)
+{
+  GString *string;
+  cairo_rectangle_int_t extents;
+
+  cairo_region_get_extents (region, &extents);
+
+  string = g_string_new (NULL);
+  g_string_append_printf (string, "{ %d, %d, %d, %d } (%d rects)",
+                          extents.x, extents.y, extents.width, extents.height,
+                          cairo_region_num_rectangles (region));
+
+  return g_string_free (string, FALSE);
+}
+
