@@ -259,6 +259,41 @@ gdk_toplevel_default_init (GdkToplevelInterface *iface)
                             FALSE,
                             G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY));
 
+  /**
+   * GdkToplevel:capabilities:
+   *
+   * The capabilities that are available for this toplevel.
+   *
+   * Since: 4.20
+   */
+  g_object_interface_install_property (iface,
+      g_param_spec_flags ("capabilities", NULL, NULL,
+                          GDK_TYPE_TOPLEVEL_CAPABILITIES,
+                          0,
+                          G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY));
+
+  /**
+   * GdkToplevel:gravity:
+   *
+   * The gravity to use when resizing a surface programmatically.
+   *
+   * Gravity describes which point of the surface we want to keep
+   * fixed (meaning that the surface will grow in the opposite direction).
+   * For example, a gravity of `GDK_GRAVITY_NORTH_EAST` means that we
+   * want to fix top right corner of the surface.
+   *
+   * This property is just a hint that may affect the result when negotiating
+   * toplevel sizes with the windowing system. It does not affect interactive
+   * resizes started with [method@Gdk.Toplevel.begin_resize].
+   *
+   * Since: 4.20
+   */
+  g_object_interface_install_property (iface,
+      g_param_spec_enum ("gravity", NULL, NULL,
+                         GDK_TYPE_GRAVITY,
+                         GDK_GRAVITY_NORTH_EAST,
+                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY));
+
   /**
    * GdkToplevel::compute-size:
    * @toplevel: a `GdkToplevel`
@@ -303,6 +338,8 @@ gdk_toplevel_install_properties (GObjectClass *object_class,
   g_object_class_override_property (object_class, first_prop + GDK_TOPLEVEL_PROP_DELETABLE, "deletable");
   g_object_class_override_property (object_class, first_prop + GDK_TOPLEVEL_PROP_FULLSCREEN_MODE, "fullscreen-mode");
   g_object_class_override_property (object_class, first_prop + GDK_TOPLEVEL_PROP_SHORTCUTS_INHIBITED, "shortcuts-inhibited");
+  g_object_class_override_property (object_class, first_prop + GDK_TOPLEVEL_PROP_CAPABILITIES, "capabilities");
+  g_object_class_override_property (object_class, first_prop + GDK_TOPLEVEL_PROP_GRAVITY, "gravity");
 
   return GDK_TOPLEVEL_NUM_PROPERTIES;
 }
@@ -828,3 +865,65 @@ gdk_toplevel_unexport_handle (GdkToplevel *toplevel,
 {
   GDK_TOPLEVEL_GET_IFACE (toplevel)->unexport_handle (toplevel, handle);
 }
+
+/**
+ * gdk_toplevel_get_capabilities:
+ * @toplevel: a `GdkToplevel`
+ *
+ * The capabilities that are available for this toplevel.
+ *
+ * Since: 4.20
+ */
+GdkToplevelCapabilities
+gdk_toplevel_get_capabilities (GdkToplevel *toplevel)
+{
+  GdkToplevelCapabilities caps;
+
+  g_return_val_if_fail (GDK_IS_TOPLEVEL (toplevel), 0);
+
+  g_object_get (toplevel, "capabilities", &caps, NULL);
+
+  return caps;
+}
+
+/**
+ * gdk_toplevel_get_gravity:
+ * @toplevel: a toplevel
+ *
+ * Returns the gravity that is used when changing the toplevel
+ * size programmatically.
+ *
+ * Returns: the gravity
+ *
+ * Since: 4.20
+ */
+GdkGravity
+gdk_toplevel_get_gravity (GdkToplevel *toplevel)
+{
+  GdkGravity gravity;
+
+  g_return_val_if_fail (GDK_IS_TOPLEVEL (toplevel), GDK_GRAVITY_NORTH_WEST);
+
+  g_object_get (toplevel, "gravity", &gravity, NULL);
+
+  return gravity;
+}
+
+/**
+ * gdk_toplevel_set_gravity:
+ * @toplevel: a toplevel
+ * @gravity: the new gravity
+ *
+ * Sets the gravity that is used when changing the toplevel
+ * size programmatically.
+ *
+ * Since: 4.20
+ */
+void
+gdk_toplevel_set_gravity (GdkToplevel *toplevel,
+                          GdkGravity   gravity)
+{
+  g_return_if_fail (GDK_IS_TOPLEVEL (toplevel));
+
+  g_object_set (toplevel, "gravity", gravity, NULL);
+}
