@@ -401,7 +401,7 @@ gdk_macos_surface_drag_begin (GdkSurface         *surface,
 
   cursor = gdk_drag_get_cursor (GDK_DRAG (drag),
                                 gdk_drag_get_selected_action (GDK_DRAG (drag)));
-  gdk_drag_set_cursor (GDK_DRAG (drag), cursor);
+  gdk_macos_drag_set_cursor (drag, cursor);
 
   if (!_gdk_macos_drag_begin (drag, content, self->window))
     {
@@ -533,7 +533,7 @@ gdk_macos_surface_class_init (GdkMacosSurfaceClass *klass)
   surface_class->set_opaque_region = gdk_macos_surface_set_opaque_region;
 
   /**
-   * GdkMacosSurface:native: (attributes org.gtk.Property.get=gdk_macos_surface_get_native_window)
+   * GdkMacosSurface:native:
    *
    * The "native" property contains the underlying NSWindow.
    */
@@ -553,29 +553,6 @@ gdk_macos_surface_init (GdkMacosSurface *self)
   self->monitors = g_ptr_array_new_with_free_func (g_object_unref);
 }
 
-gboolean
-_gdk_macos_surface_is_opaque (GdkMacosSurface *self)
-{
-  GdkSurface *surface = (GdkSurface *)self;
-
-  g_return_val_if_fail (GDK_IS_MACOS_SURFACE (self), FALSE);
-
-  if (surface->opaque_region != NULL &&
-      cairo_region_num_rectangles (surface->opaque_region) == 1)
-    {
-      cairo_rectangle_int_t extents;
-
-      cairo_region_get_extents (surface->opaque_region, &extents);
-
-      return (extents.x == 0 &&
-              extents.y == 0 &&
-              extents.width == GDK_SURFACE (self)->width &&
-              extents.height == GDK_SURFACE (self)->height);
-    }
-
-  return FALSE;
-}
-
 const char *
 _gdk_macos_surface_get_title (GdkMacosSurface *self)
 {
@@ -636,7 +613,7 @@ _gdk_macos_surface_set_native (GdkMacosSurface *self,
 }
 
 /**
- * gdk_macos_surface_get_native_window: (attributes org.gtk.Method.get_property=native)
+ * gdk_macos_surface_get_native_window:
  * @self: a #GdkMacosSurface
  *
  * Gets the underlying NSWindow used by the surface.
