@@ -18,8 +18,8 @@
 #include "config.h"
 
 #include "gdkdropprivate.h"
+#include "gdkdrop-wayland.h"
 
-#include "gdkprivate-wayland.h"
 #include "gdkcontentformats.h"
 #include "gdkdisplay-wayland.h"
 #include <glib/gi18n-lib.h>
@@ -191,20 +191,19 @@ gdk_wayland_drop_read_async (GdkDrop             *drop,
   g_task_set_priority (task, io_priority);
   g_task_set_source_tag (task, gdk_wayland_drop_read_async);
 
-#ifdef G_ENABLE_DEBUG
   if (GDK_DISPLAY_DEBUG_CHECK (gdk_drop_get_display (drop), DND))
     {
       char *s = gdk_content_formats_to_string (formats);
       gdk_debug_message ("%p: read for %s", drop, s);
       g_free (s);
     }
-#endif
   mime_type = gdk_content_formats_match_mime_type (formats,
                                                    gdk_drop_get_formats (drop));
   if (mime_type == NULL)
     {
       g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
                                _("No compatible transfer format found"));
+      g_object_unref (task);
       return;
     }
 
@@ -213,6 +212,7 @@ gdk_wayland_drop_read_async (GdkDrop             *drop,
   if (!g_unix_open_pipe (pipe_fd, O_CLOEXEC, &error))
     {
       g_task_return_error (task, error);
+      g_object_unref (task);
       return;
     }
 
@@ -220,6 +220,7 @@ gdk_wayland_drop_read_async (GdkDrop             *drop,
   stream = g_unix_input_stream_new (pipe_fd[0], TRUE);
   close (pipe_fd[1]);
   g_task_return_pointer (task, stream, g_object_unref);
+  g_object_unref (task);
 }
 
 static GInputStream *
