@@ -15,36 +15,38 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GDK_WAYLAND_MONITOR_PRIVATE_H__
-#define __GDK_WAYLAND_MONITOR_PRIVATE_H__
+#pragma once
 
 #include <glib.h>
 #include "gdkwaylandmonitor.h"
 #include "gdkmonitorprivate.h"
-#include "gdkprivate-wayland.h"
 
 struct _GdkWaylandMonitor {
   GdkMonitor parent;
 
-  guint32 id;
-  guint32 version;
+  uint32_t id;
   struct wl_output *output;
   gboolean added;
 
   struct zxdg_output_v1 *xdg_output;
-  /* Size and position, can be either from wl_output or xdg_output */
-  int32_t x;
-  int32_t y;
-  int32_t width;
-  int32_t height;
-  char *name;
-  char *description;
-  gboolean wl_output_done;
-  gboolean xdg_output_done;
+
+  /* Raw wl_output and xdg_output data */
+  GdkRectangle output_geometry;
+  GdkRectangle logical_geometry;
 };
 
 struct _GdkWaylandMonitorClass {
   GdkMonitorClass parent_class;
 };
 
-#endif
+void gdk_wayland_display_init_xdg_output (GdkWaylandDisplay *display_wayland);
+void gdk_wayland_monitor_get_xdg_output  (GdkWaylandMonitor *monitor);
+
+GdkMonitor *gdk_wayland_display_get_monitor (GdkWaylandDisplay *display,
+                                             struct wl_output  *output);
+
+void gdk_wayland_display_add_output      (GdkWaylandDisplay *display_wayland,
+                                          uint32_t           id,
+                                          struct wl_output  *output);
+void gdk_wayland_display_remove_output   (GdkWaylandDisplay *self,
+                                          uint32_t           id);
