@@ -19,10 +19,10 @@
 
 #include "gdkprimary-wayland.h"
 
+#include "gdkdisplay-wayland.h"
 #include "gdkclipboardprivate.h"
 #include "gdkcontentformats.h"
 #include <glib/gi18n-lib.h>
-#include "gdkprivate-wayland.h"
 #include "gdkprivate.h"
 
 #include <glib-unix.h>
@@ -286,7 +286,7 @@ gdk_wayland_primary_claim (GdkClipboard       *clipboard,
       const char * const *mime_types;
       gsize i, n_mime_types;
       GdkSeat *seat;
-      guint32 serial;
+      uint32_t serial;
 
       gdk_wayland_primary_discard_offer (cb);
       gdk_wayland_primary_discard_source (cb);
