@@ -510,7 +510,7 @@ typedef struct {
 static BOOL CALLBACK
 enum_monitor (HMONITOR hmonitor,
               HDC      hdc,
-              LPRECT   rect,
+              LPRECT   unused,
               LPARAM   param)
 {
   EnumMonitorData *data = (EnumMonitorData *) param;
@@ -562,6 +562,8 @@ enum_monitor (HMONITOR hmonitor,
           GdkWin32Monitor *w32mon;
           GdkMonitor *mon;
           GdkRectangle rect;
+          HMONITOR scale_hmonitor;
+          POINT pt;
           int scale;
 
           memset (&dd_monitor, 0, sizeof (dd_monitor));
@@ -661,25 +663,14 @@ enum_monitor (HMONITOR hmonitor,
           /* This is temporary, scale will be applied below */
           w32mon->work_rect = rect;
 
-          if (data->display->has_fixed_scale)
-            scale = data->display->surface_scale;
-          else
-            {
-              /* First acquire the scale using the current screen */
-              scale = gdk_win32_display_get_monitor_scale_factor (data->display, NULL, NULL);
+          /* First acquire the scale using the current screen */
+          scale = gdk_win32_display_get_monitor_scale_factor (data->display, NULL, NULL);
 
-              /* acquire the scale using the monitor which the window is nearest on Windows 8.1+ */
-              if (data->display->have_at_least_win81)
-                {
-                  HMONITOR hmonitor;
-                  POINT pt;
-
-                  pt.x = w32mon->work_rect.x + w32mon->work_rect.width / 2;
-                  pt.y = w32mon->work_rect.y + w32mon->work_rect.height / 2;
-                  hmonitor = MonitorFromPoint (pt, MONITOR_DEFAULTTONEAREST);
-                  scale = gdk_win32_display_get_monitor_scale_factor (data->display, NULL, hmonitor);
-                }
-            }
+          /* acquire the scale using the monitor which the window is nearest */
+          pt.x = w32mon->work_rect.x + w32mon->work_rect.width / 2;
+          pt.y = w32mon->work_rect.y + w32mon->work_rect.height / 2;
+          scale_hmonitor = MonitorFromPoint (pt, MONITOR_DEFAULTTONEAREST);
+          scale = gdk_win32_display_get_monitor_scale_factor (data->display, NULL, scale_hmonitor);
 
           gdk_monitor_set_scale_factor (mon, scale);
           /* Now apply the scale to the work rectangle */
