@@ -17,7 +17,7 @@
 /**
  * GskGLRenderer:
  *
- * A GL based renderer.
+ * Renders a GSK rendernode tree with OpenGL.
  *
  * See [class@Gsk.Renderer].
  *
@@ -26,8 +26,6 @@
 struct _GskGLRenderer
 {
   GskGpuRenderer parent_instance;
-
-  GskGpuImage *backbuffer;
 };
 
 struct _GskGLRendererClass
@@ -104,48 +102,28 @@ gsk_gl_renderer_restore_current (GskGpuRenderer *renderer,
     gdk_gl_context_clear_current ();
 }
 
-static void
-gsk_gl_renderer_free_backbuffer (GskGLRenderer *self)
-{
-  g_clear_object (&self->backbuffer);
-}
-
 static GskGpuImage *
 gsk_gl_renderer_get_backbuffer (GskGpuRenderer *renderer)
 {
-  GskGLRenderer *self = GSK_GL_RENDERER (renderer);
   GdkDrawContext *context;
   GdkSurface *surface;
-  double scale;
+  guint width, height;
 
   context = gsk_gpu_renderer_get_context (renderer);
   surface = gdk_draw_context_get_surface (context);
-  scale = gdk_surface_get_scale (surface);
+  gdk_draw_context_get_buffer_size (context, &width, &height);
 
-  if (self->backbuffer == NULL ||
-      !!(gsk_gpu_image_get_flags (self->backbuffer) & GSK_GPU_IMAGE_SRGB) != gdk_surface_get_gl_is_srgb (surface) ||
-      gsk_gpu_image_get_width (self->backbuffer) != ceil (gdk_surface_get_width (surface) * scale) ||
-      gsk_gpu_image_get_height (self->backbuffer) != ceil (gdk_surface_get_height (surface) * scale))
-    {
-      gsk_gl_renderer_free_backbuffer (self);
-      self->backbuffer = gsk_gl_image_new_backbuffer (GSK_GL_DEVICE (gsk_gpu_renderer_get_device (renderer)),
-                                                      GDK_GL_CONTEXT (context),
-                                                      GDK_MEMORY_DEFAULT /* FIXME */,
-                                                      gdk_surface_get_gl_is_srgb (surface),
-                                                      ceil (gdk_surface_get_width (surface) * scale),
-                                                      ceil (gdk_surface_get_height (surface) * scale));
-    }
-
-  return self->backbuffer;
+  return gsk_gl_image_new_backbuffer (GSK_GL_DEVICE (gsk_gpu_renderer_get_device (renderer)),
+                                      GDK_GL_CONTEXT (context),
+                                      GDK_MEMORY_DEFAULT /* FIXME */,
+                                      gdk_surface_get_gl_is_srgb (surface),
+                                      width,
+                                      height);
 }
 
 static void
 gsk_gl_renderer_unrealize (GskRenderer *renderer)
 {
-  GskGLRenderer *self = GSK_GL_RENDERER (renderer);
-
-  gsk_gl_renderer_free_backbuffer (self);
-
   gdk_gl_context_clear_current ();
 
   GSK_RENDERER_CLASS (gsk_gl_renderer_parent_class)->unrealize (renderer);
@@ -213,6 +191,7 @@ static gboolean
 gsk_ngl_renderer_realize (GskRenderer  *renderer,
                           GdkDisplay   *display,
                           GdkSurface   *surface,
+                          gboolean      attach,
                           GError      **error)
 {
   g_set_error_literal (error,
