@@ -2,17 +2,22 @@
 
 #include "gskgpucacheprivate.h"
 
+#include "gskgpucachedglyphprivate.h"
+#include "gskgpucachedfillprivate.h"
+#include "gskgpucachedstrokeprivate.h"
+#include "gskgpucachedprivate.h"
 #include "gskgpudeviceprivate.h"
 #include "gskgpuframeprivate.h"
 #include "gskgpuimageprivate.h"
-#include "gskgpuuploadopprivate.h"
+#ifdef GDK_RENDERING_VULKAN
+#include "gskvulkanycbcrprivate.h"
+#endif
 
 #include "gdk/gdkcolorstateprivate.h"
 #include "gdk/gdkprofilerprivate.h"
 #include "gdk/gdktextureprivate.h"
 
 #include "gsk/gskdebugprivate.h"
-#include "gsk/gskprivate.h"
 
 #define MAX_SLICES_PER_ATLAS 64
 
@@ -44,7 +49,6 @@ struct _GskGpuCache
   GHashTable *texture_cache;
   GHashTable *ccs_texture_caches[GDK_COLOR_STATE_N_IDS];
   GHashTable *tile_cache;
-  GHashTable *glyph_cache;
 
   GskGpuCachedAtlas *current_atlas;
 
@@ -52,32 +56,15 @@ struct _GskGpuCache
   /* atomic */ gsize dead_texture_pixels;
 };
 
-G_DEFINE_TYPE (GskGpuCache, gsk_gpu_cache, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (GskGpuCache, gsk_gpu_cache, G_TYPE_OBJECT)
 
 /* {{{ Cached base class */
 
-static inline void
-mark_as_stale (GskGpuCached *cached,
-               gboolean      stale)
-{
-  if (cached->stale != stale)
-    {
-      cached->stale = stale;
-
-      if (cached->atlas)
-        {
-          if (stale)
-            ((GskGpuCached *) cached->atlas)->pixels -= cached->pixels;
-          else
-            ((GskGpuCached *) cached->atlas)->pixels += cached->pixels;
-        }
-    }
-}
-
 static void
-gsk_gpu_cached_free (GskGpuCache  *self,
-                     GskGpuCached *cached)
+gsk_gpu_cached_free (GskGpuCached *cached)
 {
+  GskGpuCache *self = cached->cache;
+
   if (cached->next)
     cached->next->prev = cached->prev;
   else
@@ -87,18 +74,17 @@ gsk_gpu_cached_free (GskGpuCache  *self,
   else
     self->first_cached = cached->next;
 
-  mark_as_stale (cached, TRUE);
+  gsk_gpu_cached_set_stale (cached, TRUE);
 
-  cached->class->free (self, cached);
+  cached->class->free (cached);
 }
 
 static gboolean
-gsk_gpu_cached_should_collect (GskGpuCache  *cache,
-                               GskGpuCached *cached,
+gsk_gpu_cached_should_collect (GskGpuCached *cached,
                                gint64        cache_timeout,
                                gint64        timestamp)
 {
-  return cached->class->should_collect (cache, cached, cache_timeout, timestamp);
+  return cached->class->should_collect (cached, cache_timeout, timestamp);
 }
 
 static gpointer
@@ -110,6 +96,7 @@ gsk_gpu_cached_new_from_atlas (GskGpuCache             *cache,
 
   cached = g_malloc0 (class->size);
 
+  cached->cache = cache;
   cached->class = class;
   cached->atlas = atlas;
 
@@ -123,6 +110,15 @@ gsk_gpu_cached_new_from_atlas (GskGpuCache             *cache,
   return cached;
 }
 
+gpointer
+gsk_gpu_cached_new_from_current_atlas (GskGpuCache             *cache,
+                                       const GskGpuCachedClass *class)
+{
+  return gsk_gpu_cached_new_from_atlas (cache,
+                                        class,
+                                        cache->current_atlas);
+}
+
 gpointer
 gsk_gpu_cached_new (GskGpuCache             *cache,
                     const GskGpuCachedClass *class)
@@ -130,24 +126,11 @@ gsk_gpu_cached_new (GskGpuCache             *cache,
   return gsk_gpu_cached_new_from_atlas (cache, class, NULL);
 }
 
-static void
-gsk_gpu_cached_use (GskGpuCache  *self,
-                    GskGpuCached *cached)
+void
+gsk_gpu_cached_use (GskGpuCached *cached)
 {
-  cached->timestamp = self->timestamp;
-  mark_as_stale (cached, FALSE);
-}
-
-static inline gboolean
-gsk_gpu_cached_is_old (GskGpuCache  *self,
-                       GskGpuCached *cached,
-                       gint64        cache_timeout,
-                       gint64        timestamp)
-{
-  if (cache_timeout < 0)
-    return -1;
-  else
-    return timestamp - cached->timestamp > cache_timeout;
+  cached->timestamp = cached->cache->timestamp;
+  gsk_gpu_cached_set_stale (cached, FALSE);
 }
 
 /* }}} */
@@ -168,10 +151,10 @@ struct _GskGpuCachedAtlas
 };
 
 static void
-gsk_gpu_cached_atlas_free (GskGpuCache  *cache,
-                           GskGpuCached *cached)
+gsk_gpu_cached_atlas_free (GskGpuCached *cached)
 {
   GskGpuCachedAtlas *self = (GskGpuCachedAtlas *) cached;
+  GskGpuCache *cache = cached->cache;
   GskGpuCached *c, *next;
 
   /* Free all remaining glyphs on this atlas */
@@ -179,7 +162,7 @@ gsk_gpu_cached_atlas_free (GskGpuCache  *cache,
     {
       next = c->next;
       if (c->atlas == self)
-        gsk_gpu_cached_free (cache, c);
+        gsk_gpu_cached_free (c);
     }
 
   if (cache->current_atlas == self)
@@ -191,15 +174,14 @@ gsk_gpu_cached_atlas_free (GskGpuCache  *cache,
 }
 
 static gboolean
-gsk_gpu_cached_atlas_should_collect (GskGpuCache  *cache,
-                                     GskGpuCached *cached,
+gsk_gpu_cached_atlas_should_collect (GskGpuCached *cached,
                                      gint64        cache_timeout,
                                      gint64        timestamp)
 {
   GskGpuCachedAtlas *self = (GskGpuCachedAtlas *) cached;
 
-  if (cache->current_atlas == self &&
-      gsk_gpu_cached_is_old (cache, cached, cache_timeout * ATLAS_TIMEOUT_SCALE, timestamp) &&
+  if (cached->cache->current_atlas == self &&
+      gsk_gpu_cached_is_old (cached, cache_timeout * ATLAS_TIMEOUT_SCALE, timestamp) &&
       cached->pixels == 0)
     return TRUE;
 
@@ -332,7 +314,7 @@ gsk_gpu_cache_get_atlas_image (GskGpuCache *self)
   return self->current_atlas->image;
 }
 
-static GskGpuImage *
+GskGpuImage *
 gsk_gpu_cache_add_atlas_image (GskGpuCache      *self,
                                gsize             width,
                                gsize             height,
@@ -346,7 +328,7 @@ gsk_gpu_cache_add_atlas_image (GskGpuCache      *self,
 
   if (gsk_gpu_cached_atlas_allocate (self->current_atlas, width, height, out_x, out_y))
     {
-      gsk_gpu_cached_use (self, (GskGpuCached *) self->current_atlas);
+      gsk_gpu_cached_use ((GskGpuCached *) self->current_atlas);
       return self->current_atlas->image;
     }
 
@@ -354,7 +336,7 @@ gsk_gpu_cache_add_atlas_image (GskGpuCache      *self,
 
   if (gsk_gpu_cached_atlas_allocate (self->current_atlas, width, height, out_x, out_y))
     {
-      gsk_gpu_cached_use (self, (GskGpuCached *) self->current_atlas);
+      gsk_gpu_cached_use ((GskGpuCached *) self->current_atlas);
       return self->current_atlas->image;
     }
 
@@ -405,10 +387,10 @@ gsk_gpu_cache_get_texture_hash_table (GskGpuCache   *cache,
 }
 
 static void
-gsk_gpu_cached_texture_free (GskGpuCache  *cache,
-                             GskGpuCached *cached)
+gsk_gpu_cached_texture_free (GskGpuCached *cached)
 {
   GskGpuCachedTexture *self = (GskGpuCachedTexture *) cached;
+  GskGpuCache *cache = cached->cache;
   GHashTable *texture_cache;
   gpointer key, value;
 
@@ -444,14 +426,13 @@ gsk_gpu_cached_texture_is_invalid (GskGpuCachedTexture *self)
 }
 
 static gboolean
-gsk_gpu_cached_texture_should_collect (GskGpuCache *cache,
-                                       GskGpuCached *cached,
+gsk_gpu_cached_texture_should_collect (GskGpuCached *cached,
                                        gint64        cache_timeout,
                                        gint64        timestamp)
 {
   GskGpuCachedTexture *self = (GskGpuCachedTexture *) cached;
 
-  return gsk_gpu_cached_is_old (cache, cached, cache_timeout, timestamp) ||
+  return gsk_gpu_cached_is_old (cached, cache_timeout, timestamp) ||
          gsk_gpu_cached_texture_is_invalid (self);
 }
 
@@ -555,10 +536,10 @@ struct _GskGpuCachedTile
 };
 
 static void
-gsk_gpu_cached_tile_free (GskGpuCache  *cache,
-                          GskGpuCached *cached)
+gsk_gpu_cached_tile_free (GskGpuCached *cached)
 {
   GskGpuCachedTile *self = (GskGpuCachedTile *) cached;
+  GskGpuCache *cache = cached->cache;
   gpointer key, value;
 
   g_clear_object (&self->image);
@@ -592,14 +573,13 @@ gsk_gpu_cached_tile_is_invalid (GskGpuCachedTile *self)
 }
 
 static gboolean
-gsk_gpu_cached_tile_should_collect (GskGpuCache  *cache,
-                                    GskGpuCached *cached,
+gsk_gpu_cached_tile_should_collect (GskGpuCached *cached,
                                     gint64        cache_timeout,
                                     gint64        timestamp)
 {
   GskGpuCachedTile *self = (GskGpuCachedTile *) cached;
 
-  return gsk_gpu_cached_is_old (cache, cached, cache_timeout, timestamp) ||
+  return gsk_gpu_cached_is_old (cached, cache_timeout, timestamp) ||
          gsk_gpu_cached_tile_is_invalid (self);
 }
 
@@ -708,7 +688,7 @@ gsk_gpu_cache_lookup_tile (GskGpuCache      *self,
   if (tile == NULL)
     return NULL;
 
-  gsk_gpu_cached_use (self, (GskGpuCached *) tile);
+  gsk_gpu_cached_use ((GskGpuCached *) tile);
 
   *out_color_state = tile->color_state;
 
@@ -734,90 +714,9 @@ gsk_gpu_cache_cache_tile (GskGpuCache      *self,
                                   image,
                                   color_state);
 
-  gsk_gpu_cached_use (self, (GskGpuCached *) tile);
+  gsk_gpu_cached_use ((GskGpuCached *) tile);
 }
 
-/* }}} */
-/* {{{ CachedGlyph */
-
-struct _GskGpuCachedGlyph
-{
-  GskGpuCached parent;
-
-  PangoFont *font;
-  PangoGlyph glyph;
-  GskGpuGlyphLookupFlags flags;
-  float scale;
-
-  GskGpuImage *image;
-  graphene_rect_t bounds;
-  graphene_point_t origin;
-};
-
-static void
-gsk_gpu_cached_glyph_free (GskGpuCache  *cache,
-                           GskGpuCached *cached)
-{
-  GskGpuCachedGlyph *self = (GskGpuCachedGlyph *) cached;
-
-  g_hash_table_remove (cache->glyph_cache, self);
-
-  g_object_unref (self->font);
-  g_object_unref (self->image);
-
-  g_free (self);
-}
-
-static gboolean
-gsk_gpu_cached_glyph_should_collect (GskGpuCache  *cache,
-                                     GskGpuCached *cached,
-                                     gint64        cache_timeout,
-                                     gint64        timestamp)
-{
-  if (gsk_gpu_cached_is_old (cache, cached, cache_timeout, timestamp))
-    {
-      if (cached->atlas)
-        mark_as_stale (cached, TRUE);
-      else
-        return TRUE;
-    }
-
-  /* Glyphs are only collected when their atlas is freed */
-  return FALSE;
-}
-
-static guint
-gsk_gpu_cached_glyph_hash (gconstpointer data)
-{
-  const GskGpuCachedGlyph *glyph = data;
-
-  return GPOINTER_TO_UINT (glyph->font) ^
-         glyph->glyph ^
-         (glyph->flags << 24) ^
-         ((guint) glyph->scale * PANGO_SCALE);
-}
-
-static gboolean
-gsk_gpu_cached_glyph_equal (gconstpointer v1,
-                            gconstpointer v2)
-{
-  const GskGpuCachedGlyph *glyph1 = v1;
-  const GskGpuCachedGlyph *glyph2 = v2;
-
-  return glyph1->font == glyph2->font
-      && glyph1->glyph == glyph2->glyph
-      && glyph1->flags == glyph2->flags
-      && glyph1->scale == glyph2->scale;
-}
-
-static const GskGpuCachedClass GSK_GPU_CACHED_GLYPH_CLASS =
-{
-  sizeof (GskGpuCachedGlyph),
-  "Glyph",
-  gsk_gpu_cached_glyph_free,
-  gsk_gpu_cached_glyph_should_collect
-};
-
 /* }}} */
 /* {{{ GskGpuCache */
 
@@ -924,8 +823,8 @@ gsk_gpu_cache_gc (GskGpuCache *self,
   for (cached = self->last_cached; cached != NULL; cached = prev)
     {
       prev = cached->prev;
-      if (gsk_gpu_cached_should_collect (self, cached, cache_timeout, timestamp))
-        gsk_gpu_cached_free (self, cached);
+      if (gsk_gpu_cached_should_collect (cached, cache_timeout, timestamp))
+        gsk_gpu_cached_free (cached);
       else
         is_empty &= cached->stale;
     }
@@ -970,7 +869,7 @@ gsk_gpu_cache_clear_cache (GskGpuCache *self)
 
   /* We clear the cache from the end so glyphs get freed before their atlas */
   while (self->last_cached)
-    gsk_gpu_cached_free (self, self->last_cached);
+    gsk_gpu_cached_free (self->last_cached);
 
   g_assert (self->last_cached == NULL);
 }
@@ -981,7 +880,15 @@ gsk_gpu_cache_dispose (GObject *object)
   GskGpuCache *self = GSK_GPU_CACHE (object);
 
   gsk_gpu_cache_clear_cache (self);
-  g_hash_table_unref (self->glyph_cache);
+
+  gsk_gpu_cached_stroke_finish_cache (self);
+  gsk_gpu_cached_fill_finish_cache (self);
+
+#ifdef GDK_RENDERING_VULKAN
+  gsk_vulkan_ycbcr_finish_cache (self);
+#endif
+  gsk_gpu_cached_glyph_finish_cache (self);
+
   g_clear_pointer (&self->tile_cache, g_hash_table_unref);
   g_hash_table_unref (self->texture_cache);
 
@@ -1010,10 +917,15 @@ gsk_gpu_cache_class_init (GskGpuCacheClass *klass)
 static void
 gsk_gpu_cache_init (GskGpuCache *self)
 {
-  self->glyph_cache = g_hash_table_new (gsk_gpu_cached_glyph_hash,
-                                        gsk_gpu_cached_glyph_equal);
   self->texture_cache = g_hash_table_new (g_direct_hash,
                                           g_direct_equal);
+  
+  gsk_gpu_cached_glyph_init_cache (self);
+#ifdef GDK_RENDERING_VULKAN
+  gsk_vulkan_ycbcr_init_cache (self);
+#endif
+  gsk_gpu_cached_fill_init_cache (self);
+  gsk_gpu_cached_stroke_init_cache (self);
 }
 
 GskGpuImage *
@@ -1037,7 +949,7 @@ gsk_gpu_cache_lookup_texture_image (GskGpuCache   *self,
   if (!cache || !cache->image || gsk_gpu_cached_texture_is_invalid (cache))
     return NULL;
 
-  gsk_gpu_cached_use (self, (GskGpuCached *) cache);
+  gsk_gpu_cached_use ((GskGpuCached *) cache);
 
   return g_object_ref (cache->image);
 }
@@ -1054,106 +966,7 @@ gsk_gpu_cache_cache_texture_image (GskGpuCache   *self,
   if (cache == NULL)
     return;
 
-  gsk_gpu_cached_use (self, (GskGpuCached *) cache);
-}
-
-GskGpuImage *
-gsk_gpu_cache_lookup_glyph_image (GskGpuCache            *self,
-                                  GskGpuFrame            *frame,
-                                  PangoFont              *font,
-                                  PangoGlyph              glyph,
-                                  GskGpuGlyphLookupFlags  flags,
-                                  float                   scale,
-                                  graphene_rect_t        *out_bounds,
-                                  graphene_point_t       *out_origin)
-{
-  GskGpuCachedGlyph lookup = {
-    .font = font,
-    .glyph = glyph,
-    .flags = flags,
-    .scale = scale
-  };
-  GskGpuCachedGlyph *cache;
-  PangoRectangle ink_rect;
-  graphene_rect_t rect;
-  graphene_point_t origin;
-  GskGpuImage *image;
-  gsize atlas_x, atlas_y, padding;
-  float subpixel_x, subpixel_y;
-  PangoFont *scaled_font;
-
-  cache = g_hash_table_lookup (self->glyph_cache, &lookup);
-  if (cache)
-    {
-      gsk_gpu_cached_use (self, (GskGpuCached *) cache);
-
-      *out_bounds = cache->bounds;
-      *out_origin = cache->origin;
-      return cache->image;
-    }
-
-  scaled_font = gsk_reload_font (font, scale, CAIRO_HINT_METRICS_DEFAULT, CAIRO_HINT_STYLE_DEFAULT, CAIRO_ANTIALIAS_DEFAULT);
-
-  subpixel_x = (flags & 3) / 4.f;
-  subpixel_y = ((flags >> 2) & 3) / 4.f;
-  pango_font_get_glyph_extents (scaled_font, glyph, &ink_rect, NULL);
-  origin.x = floor (ink_rect.x * 1.0 / PANGO_SCALE + subpixel_x);
-  origin.y = floor (ink_rect.y * 1.0 / PANGO_SCALE + subpixel_y);
-  rect.size.width = ceil ((ink_rect.x + ink_rect.width) * 1.0 / PANGO_SCALE + subpixel_x) - origin.x;
-  rect.size.height = ceil ((ink_rect.y + ink_rect.height) * 1.0 / PANGO_SCALE + subpixel_y) - origin.y;
-  padding = 1;
-
-  image = gsk_gpu_cache_add_atlas_image (self,
-                                         rect.size.width + 2 * padding, rect.size.height + 2 * padding,
-                                         &atlas_x, &atlas_y);
-  if (image)
-    {
-      g_object_ref (image);
-      rect.origin.x = atlas_x + padding;
-      rect.origin.y = atlas_y + padding;
-      cache = gsk_gpu_cached_new_from_atlas (self, &GSK_GPU_CACHED_GLYPH_CLASS, self->current_atlas);
-    }
-  else
-    {
-      image = gsk_gpu_device_create_upload_image (self->device, FALSE, GDK_MEMORY_DEFAULT, FALSE, rect.size.width, rect.size.height),
-      rect.origin.x = 0;
-      rect.origin.y = 0;
-      padding = 0;
-      cache = gsk_gpu_cached_new (self, &GSK_GPU_CACHED_GLYPH_CLASS);
-    }
-
-  cache->font = g_object_ref (font);
-  cache->glyph = glyph;
-  cache->flags = flags;
-  cache->scale = scale;
-  cache->bounds = rect;
-  cache->image = image;
-  cache->origin = GRAPHENE_POINT_INIT (- origin.x + subpixel_x,
-                                       - origin.y + subpixel_y);
-  ((GskGpuCached *) cache)->pixels = (rect.size.width + 2 * padding) * (rect.size.height + 2 * padding);
-
-  gsk_gpu_upload_glyph_op (frame,
-                           cache->image,
-                           scaled_font,
-                           glyph,
-                           &(cairo_rectangle_int_t) {
-                               .x = rect.origin.x - padding,
-                               .y = rect.origin.y - padding,
-                               .width = rect.size.width + 2 * padding,
-                               .height = rect.size.height + 2 * padding,
-                           },
-                           &GRAPHENE_POINT_INIT (cache->origin.x + padding,
-                                                 cache->origin.y + padding));
-
-  g_hash_table_insert (self->glyph_cache, cache, cache);
-  gsk_gpu_cached_use (self, (GskGpuCached *) cache);
-
-  *out_bounds = cache->bounds;
-  *out_origin = cache->origin;
-
-  g_object_unref (scaled_font);
-
-  return cache->image;
+  gsk_gpu_cached_use ((GskGpuCached *) cache);
 }
 
 GskGpuCache *
@@ -1167,5 +980,11 @@ gsk_gpu_cache_new (GskGpuDevice *device)
   return self;
 }
 
+GskGpuCachePrivate *
+gsk_gpu_cache_get_private (GskGpuCache *self)
+{
+  return gsk_gpu_cache_get_instance_private (self);
+}
+
 /* }}} */
 /* vim:set foldmethod=marker: */
