@@ -5,6 +5,10 @@
 
 #include "gdk/gdkdmabufprivate.h"
 
+#ifdef GDK_WINDOWING_WIN32
+#include <gdk/win32/gdkwin32.h>
+#endif
+
 G_BEGIN_DECLS
 
 #define GSK_TYPE_VULKAN_IMAGE (gsk_vulkan_image_get_type ())
@@ -14,6 +18,7 @@ G_DECLARE_FINAL_TYPE (GskVulkanImage, gsk_vulkan_image, GSK, VULKAN_IMAGE, GskGp
 GskGpuImage *           gsk_vulkan_image_new_for_swapchain              (GskVulkanDevice        *device,
                                                                          VkImage                 image,
                                                                          VkFormat                format,
+                                                                         GdkMemoryFormat         memory_format,
                                                                          gsize                   width,
                                                                          gsize                   height);
 
@@ -23,27 +28,46 @@ GskGpuImage *           gsk_vulkan_image_new_for_atlas                  (GskVulk
 GskGpuImage *           gsk_vulkan_image_new_for_offscreen              (GskVulkanDevice        *device,
                                                                          gboolean                with_mipmap,
                                                                          GdkMemoryFormat         preferred_format,
+                                                                         gboolean                try_srgb,
                                                                          gsize                   width,
                                                                          gsize                   height);
 GskGpuImage *           gsk_vulkan_image_new_for_upload                 (GskVulkanDevice        *device,
                                                                          gboolean                with_mipmap,
                                                                          GdkMemoryFormat         format,
+                                                                         GskGpuConversion        conv,
                                                                          gsize                   width,
                                                                          gsize                   height);
 #ifdef HAVE_DMABUF
 GskGpuImage *           gsk_vulkan_image_new_dmabuf                     (GskVulkanDevice        *device,
                                                                          GdkMemoryFormat         format,
+                                                                         gboolean                try_srgb,
                                                                          gsize                   width,
                                                                          gsize                   height);
 GskGpuImage *           gsk_vulkan_image_new_for_dmabuf                 (GskVulkanDevice        *device,
-                                                                         GdkTexture             *texture);
-GdkTexture *            gsk_vulkan_image_to_dmabuf_texture              (GskVulkanImage         *self);
+                                                                         gsize                   width,
+                                                                         gsize                   height,
+                                                                         const GdkDmabuf        *dmabuf,
+                                                                         gboolean                premultiplied,
+                                                                         GskGpuConversion        conv);
+GdkTexture *            gsk_vulkan_image_to_dmabuf_texture              (GskVulkanImage         *self,
+                                                                         GdkColorState          *color_state);
+#endif
+#ifdef GDK_WINDOWING_WIN32
+GskGpuImage *           gsk_vulkan_image_new_for_d3d12resource          (GskVulkanDevice        *device,
+                                                                         ID3D12Resource         *resource,
+                                                                         HANDLE                  resource_handle,
+                                                                         ID3D12Fence            *fence,
+                                                                         HANDLE                  fence_handle,
+                                                                         guint64                 fence_wait,
+                                                                         gboolean                premultiplied);
 #endif
 
 guchar *                gsk_vulkan_image_get_data                       (GskVulkanImage         *self,
-                                                                         gsize                  *out_stride);
+                                                                         GdkMemoryLayout        *out_layout);
 
-VkSampler               gsk_vulkan_image_get_vk_sampler                 (GskVulkanImage         *self);
+GskVulkanYcbcr *        gsk_vulkan_image_get_ycbcr                      (GskVulkanImage         *self);
+VkDescriptorSet         gsk_vulkan_image_get_vk_descriptor_set          (GskVulkanImage         *self,
+                                                                         GskGpuSampler           sampler);
 VkPipelineStageFlags    gsk_vulkan_image_get_vk_pipeline_stage          (GskVulkanImage         *self);
 VkImageLayout           gsk_vulkan_image_get_vk_image_layout            (GskVulkanImage         *self);
 VkAccessFlags           gsk_vulkan_image_get_vk_access                  (GskVulkanImage         *self);
@@ -67,12 +91,5 @@ VkFormat                gsk_vulkan_image_get_vk_format                  (GskVulk
 VkFramebuffer           gsk_vulkan_image_get_vk_framebuffer             (GskVulkanImage         *self,
                                                                          VkRenderPass            pass);
 
-static inline guint
-gsk_vulkan_mipmap_levels (gsize width,
-                          gsize height)
-{
-  return g_bit_nth_msf (MAX (MAX (width, height) - 1, 1), -1) + 1;
-}
-
 G_END_DECLS
 
