@@ -7,20 +7,20 @@ G_BEGIN_DECLS
 
 typedef enum {
   GSK_DEBUG_RENDERER              = 1 <<  0,
-  GSK_DEBUG_SHADERS               = 1 <<  2,
-  GSK_DEBUG_VULKAN                = 1 <<  3,
-  GSK_DEBUG_FALLBACK              = 1 <<  4,
-  GSK_DEBUG_GLYPH_CACHE           = 1 <<  5,
-  GSK_DEBUG_VERBOSE               = 1 <<  6,
+  GSK_DEBUG_SHADERS               = 1 <<  1,
+  GSK_DEBUG_VULKAN                = 1 <<  2,
+  GSK_DEBUG_FALLBACK              = 1 <<  3,
+  GSK_DEBUG_CACHE                 = 1 <<  4,
+  GSK_DEBUG_VERBOSE               = 1 <<  5,
   /* flags below may affect behavior */
-  GSK_DEBUG_GEOMETRY              = 1 <<  7,
-  GSK_DEBUG_FULL_REDRAW           = 1 <<  8,
-  GSK_DEBUG_STAGING               = 1 << 10,
-  GSK_DEBUG_OFFLOAD_DISABLE       = 1 << 11,
-  GSK_DEBUG_CAIRO                 = 1 << 12,
+  GSK_DEBUG_GEOMETRY              = 1 <<  6,
+  GSK_DEBUG_FULL_REDRAW           = 1 <<  7,
+  GSK_DEBUG_STAGING               = 1 <<  8,
+  GSK_DEBUG_CAIRO                 = 1 <<  9,
+  GSK_DEBUG_OCCLUSION             = 1 << 10,
 } GskDebugFlags;
 
-#define GSK_DEBUG_ANY ((1 << 13) - 1)
+#define GSK_DEBUG_ANY ((1 << 11) - 1)
 
 GskDebugFlags gsk_get_debug_flags (void);
 void          gsk_set_debug_flags (GskDebugFlags flags);
