@@ -24,7 +24,7 @@
 /**
  * GskStroke:
  *
- * Collects the parameters that influence the operation of stroking a path.
+ * Collects the parameters that are needed when stroking a path.
  *
  * Since: 4.14
  */
@@ -47,7 +47,7 @@ gsk_stroke_new (float line_width)
 {
   GskStroke *self;
 
-  g_return_val_if_fail (line_width > 0, NULL);
+  g_return_val_if_fail (line_width >= 0, NULL);
 
   self = g_new0 (GskStroke, 1);
 
@@ -209,7 +209,7 @@ gsk_stroke_equal (gconstpointer stroke1,
  *
  * Sets the line width to be used when stroking.
  *
- * The line width must be > 0.
+ * The line width must be >= 0.
  *
  * Since: 4.14
  */
@@ -218,7 +218,7 @@ gsk_stroke_set_line_width (GskStroke *self,
                            float      line_width)
 {
   g_return_if_fail (self != NULL);
-  g_return_if_fail (line_width > 0);
+  g_return_if_fail (line_width >= 0);
 
   self->line_width = line_width;
 }
