@@ -35,14 +35,6 @@ xml += '''
     <file>theme/Empty/gtk.css</file>
 
     <file>theme/Default/gtk.css</file>
-    <file alias='theme/Default-dark/gtk.css'>theme/Default/gtk-dark.css</file>
-    <file alias='theme/Default-hc/gtk.css'>theme/Default/gtk-hc.css</file>
-    <file alias='theme/Default-hc-dark/gtk.css'>theme/Default/gtk-hc-dark.css</file>
-
-    <file>theme/Default/gtk-light.css</file>
-    <file>theme/Default/gtk-dark.css</file>
-    <file>theme/Default/gtk-hc.css</file>
-    <file>theme/Default/gtk-hc-dark.css</file>
     <file>theme/Default/Default-light.css</file>
     <file>theme/Default/Default-dark.css</file>
     <file>theme/Default/Default-hc.css</file>
