@@ -25,10 +25,8 @@
 #include "gtknative.h"
 
 #include <gdk/wayland/gdkwayland.h>
-#include <gdk/wayland/gdkdisplay-wayland.h>
-#include <gdk/wayland/gdksurface-wayland.h>
 #include <gdk/wayland/gdktoplevel-wayland-private.h>
-#include <gdk/wayland/idle-inhibit-unstable-v1-client-protocol.h>
+
 
 typedef struct
 {
@@ -156,7 +154,7 @@ gtk_application_impl_wayland_inhibit (GtkApplicationImpl         *impl,
   inhibitor->flags = flags;
   wayland->inhibitors = g_slist_prepend (wayland->inhibitors, inhibitor);
 
-  if (flags & GTK_APPLICATION_INHIBIT_IDLE && impl->application == gtk_window_get_application (window))
+  if (flags & GTK_APPLICATION_INHIBIT_IDLE && window && impl->application == gtk_window_get_application (window))
     {
       surface = gtk_native_get_surface (GTK_NATIVE (window));
       if (GDK_IS_WAYLAND_TOPLEVEL (surface))
