@@ -22,13 +22,12 @@
 #include "gtkbookmarklist.h"
 
 #include "gtksettings.h"
-#include "gtkintl.h"
 #include "gtkprivate.h"
 
 /**
  * GtkBookmarkList:
  *
- * `GtkBookmarkList` is a list model that wraps `GBookmarkFile`.
+ * A list model that wraps `GBookmarkFile`.
  *
  * It presents a `GListModel` and fills it asynchronously with the
  * `GFileInfo`s returned from that function.
@@ -219,7 +218,7 @@ gtk_bookmark_list_class_init (GtkBookmarkListClass *class)
   gobject_class->dispose = gtk_bookmark_list_dispose;
 
   /**
-   * GtkBookmarkList:filename: (attributes org.gtk.Property.get=gtk_bookmark_list_get_filename)
+   * GtkBookmarkList:filename:
    *
    * The bookmark file to load.
    */
@@ -228,7 +227,7 @@ gtk_bookmark_list_class_init (GtkBookmarkListClass *class)
                            NULL,
                            GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_EXPLICIT_NOTIFY);
   /**
-   * GtkBookmarkList:attributes: (attributes org.gtk.Property.get=gtk_bookmark_list_get_attributes org.gtk.Property.set=gtk_bookmark_list_set_attributes)
+   * GtkBookmarkList:attributes:
    *
    * The attributes to query.
    */
@@ -238,7 +237,7 @@ gtk_bookmark_list_class_init (GtkBookmarkListClass *class)
                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkBookmarkList:io-priority: (attributes org.gtk.Property.get=gtk_bookmark_list_get_io_priority org.gtk.Property.set=gtk_bookmark_list_set_io_priority)
+   * GtkBookmarkList:io-priority:
    *
    * Priority used when loading.
    */
@@ -260,7 +259,7 @@ gtk_bookmark_list_class_init (GtkBookmarkListClass *class)
                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkBookmarkList:loading: (attributes org.gtk.Property.get=gtk_bookmark_list_is_loading)
+   * GtkBookmarkList:loading: (getter is_loading)
    *
    * %TRUE if files are being loaded.
    */
@@ -475,7 +474,7 @@ gtk_bookmark_list_set_filename (GtkBookmarkList *self,
 }
 
 /**
- * gtk_bookmark_list_get_filename: (attributes org.gtk.Method.get_property=filename)
+ * gtk_bookmark_list_get_filename:
  * @self: a `GtkBookmarkList`
  *
  * Returns the filename of the bookmark file that
@@ -511,7 +510,7 @@ gtk_bookmark_list_new (const char *filename,
 }
 
 /**
- * gtk_bookmark_list_set_attributes: (attributes org.gtk.Method.set_property=attributes)
+ * gtk_bookmark_list_set_attributes:
  * @self: a `GtkBookmarkList`
  * @attributes: (nullable): the attributes to enumerate
  *
@@ -542,7 +541,7 @@ gtk_bookmark_list_set_attributes (GtkBookmarkList *self,
 }
 
 /**
- * gtk_bookmark_list_get_attributes: (attributes org.gtk.Method.get_property=attributes)
+ * gtk_bookmark_list_get_attributes:
  * @self: a `GtkBookmarkList`
  *
  * Gets the attributes queried on the children.
@@ -558,7 +557,7 @@ gtk_bookmark_list_get_attributes (GtkBookmarkList *self)
 }
 
 /**
- * gtk_bookmark_list_set_io_priority: (attributes org.gtk.Method.set_property=io-priority)
+ * gtk_bookmark_list_set_io_priority:
  * @self: a `GtkBookmarkList`
  * @io_priority: IO priority to use
  *
@@ -581,7 +580,7 @@ gtk_bookmark_list_set_io_priority (GtkBookmarkList *self,
 }
 
 /**
- * gtk_bookmark_list_get_io_priority: (attributes org.gtk.Method.get_property=io-priority)
+ * gtk_bookmark_list_get_io_priority:
  * @self: a `GtkBookmarkList`
  *
  * Gets the IO priority to use while loading file.
@@ -597,7 +596,7 @@ gtk_bookmark_list_get_io_priority (GtkBookmarkList *self)
 }
 
 /**
- * gtk_bookmark_list_is_loading: (attributes org.gtk.Method.get_property=loading)
+ * gtk_bookmark_list_is_loading: (get-property loading)
  * @self: a `GtkBookmarkList`
  *
  * Returns %TRUE if the files are currently being loaded.
