@@ -28,29 +28,36 @@
 /**
  * GtkCalendar:
  *
- * `GtkCalendar` is a widget that displays a Gregorian calendar, one month
- * at a time.
+ * Displays a Gregorian calendar, one month at a time.
  *
- * ![An example GtkCalendar](calendar.png)
+ * <picture>
+ *   <source srcset="calendar-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkCalendar" src="calendar.png">
+ * </picture>
  *
  * A `GtkCalendar` can be created with [ctor@Gtk.Calendar.new].
  *
- * The date that is currently displayed can be altered with
- * [method@Gtk.Calendar.select_day].
+ * The selected date can be retrieved from a `GtkCalendar` using
+ * [method@Gtk.Calendar.get_date].
+ * It can be altered with [method@Gtk.Calendar.set_date].
  *
  * To place a visual marker on a particular day, use
  * [method@Gtk.Calendar.mark_day] and to remove the marker,
  * [method@Gtk.Calendar.unmark_day]. Alternative, all
  * marks can be cleared with [method@Gtk.Calendar.clear_marks].
  *
- * The selected date can be retrieved from a `GtkCalendar` using
- * [method@Gtk.Calendar.get_date].
- *
  * Users should be aware that, although the Gregorian calendar is the
  * legal calendar in most countries, it was adopted progressively
  * between 1582 and 1929. Display before these dates is likely to be
  * historically incorrect.
  *
+ * # Shortcuts and Gestures
+ *
+ * `GtkCalendar` supports the following gestures:
+ *
+ * - Scrolling up or down will switch to the previous or next month.
+ * - Date strings can be dropped for setting the current day.
+ *
  * # CSS nodes
  *
  * ```
@@ -123,6 +130,10 @@
 #include "gtkstack.h"
 #include "gtkgrid.h"
 
+/* GDateTime is used, from 0001-01-01 to 9999-12-31 */
+static const int YEAR_MIN = 1;
+static const int YEAR_MAX = 9999;
+
 static const guint month_length[2][13] =
 {
   { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
@@ -185,6 +196,7 @@ enum {
 enum
 {
   PROP_0,
+  PROP_DATE,
   PROP_YEAR,
   PROP_MONTH,
   PROP_DAY,
@@ -264,6 +276,10 @@ static gboolean gtk_calendar_key_controller_key_pressed (GtkEventControllerKey *
 static void     gtk_calendar_focus_controller_focus     (GtkEventController    *controller,
                                                          GtkWidget             *widget);
 
+static void calendar_select_day_internal (GtkCalendar *calendar,
+                                          GDateTime   *date,
+                                          gboolean     emit_day_signal);
+
 static void calendar_invalidate_day     (GtkCalendar *widget,
                                          int        row,
                                          int        col);
@@ -331,7 +347,7 @@ gtk_calendar_drag_drop (GtkDropTarget  *dest,
                                     0, 0, 0);
   g_date_free (date);
 
-  gtk_calendar_select_day (calendar, datetime);
+  calendar_select_day_internal (calendar, datetime, TRUE);
   g_date_time_unref (datetime);
 
   return TRUE;
@@ -362,12 +378,28 @@ gtk_calendar_class_init (GtkCalendarClass *class)
   gobject_class->set_property = gtk_calendar_set_property;
   gobject_class->get_property = gtk_calendar_get_property;
 
+  /**
+   * GtkCalendar:date:
+   *
+   * The selected date.
+   *
+   * This property gets initially set to the current date.
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_DATE,
+                                   g_param_spec_boxed ("date", NULL, NULL,
+                                                       G_TYPE_DATE_TIME,
+                                                       G_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
+
   /**
    * GtkCalendar:year:
    *
    * The selected year.
    *
    * This property gets initially set to the current year.
+   *
+   * Deprecated: 4.20: This property will be removed in GTK 5.
+   *   Use [property@Calendar:date] instead.
    */
   g_object_class_install_property (gobject_class,
                                    PROP_YEAR,
@@ -381,6 +413,9 @@ gtk_calendar_class_init (GtkCalendarClass *class)
    * The selected month (as a number between 0 and 11).
    *
    * This property gets initially set to the current month.
+   *
+   * Deprecated: 4.20: This property will be removed in GTK 5.
+   *   Use [property@Calendar:date] instead.
    */
   g_object_class_install_property (gobject_class,
                                    PROP_MONTH,
@@ -392,6 +427,9 @@ gtk_calendar_class_init (GtkCalendarClass *class)
    * GtkCalendar:day:
    *
    * The selected day (as a number between 1 and 31).
+   *
+   * Deprecated: 4.20: This property will be removed in GTK 5.
+   *   Use [property@Calendar:date] instead.
    */
   g_object_class_install_property (gobject_class,
                                    PROP_DAY,
@@ -400,7 +438,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
                                                      G_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkCalendar:show-heading: (attributes org.gtk.Property.get=gtk_calendar_get_show_heading org.gtk.Property.set=gtk_calendar_set_show_heading)
+   * GtkCalendar:show-heading:
    *
    * Determines whether a heading is displayed.
    */
@@ -411,7 +449,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
                                                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkCalendar:show-day-names: (attributes org.gtk.Property.get=gtk_calendar_get_show_day_names org.gtk.Property.set=gtk_calendar_set_show_day_names)
+   * GtkCalendar:show-day-names:
    *
    * Determines whether day names are displayed.
    */
@@ -421,7 +459,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
                                                          TRUE,
                                                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
   /**
-   * GtkCalendar:show-week-numbers: (attributes org.gtk.Property.get=gtk_calendar_get_show_week_numbers org.gtk.Property.set=gtk_calendar_set_show_week_numbers)
+   * GtkCalendar:show-week-numbers:
    *
    * Determines whether week numbers are displayed.
    */
@@ -450,7 +488,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
    * GtkCalendar::prev-month:
    * @calendar: the object which received the signal.
    *
-   * Emitted when the user switched to the previous month.
+   * Emitted when the user switches to the previous month.
    */
   gtk_calendar_signals[PREV_MONTH_SIGNAL] =
     g_signal_new (I_("prev-month"),
@@ -465,7 +503,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
    * GtkCalendar::next-month:
    * @calendar: the object which received the signal.
    *
-   * Emitted when the user switched to the next month.
+   * Emitted when the user switches to the next month.
    */
   gtk_calendar_signals[NEXT_MONTH_SIGNAL] =
     g_signal_new (I_("next-month"),
@@ -480,7 +518,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
    * GtkCalendar::prev-year:
    * @calendar: the object which received the signal.
    *
-   * Emitted when user switched to the previous year.
+   * Emitted when user switches to the previous year.
    */
   gtk_calendar_signals[PREV_YEAR_SIGNAL] =
     g_signal_new (I_("prev-year"),
@@ -495,7 +533,7 @@ gtk_calendar_class_init (GtkCalendarClass *class)
    * GtkCalendar::next-year:
    * @calendar: the object which received the signal.
    *
-   * Emitted when user switched to the next year.
+   * Emitted when user switches to the next year.
    */
   gtk_calendar_signals[NEXT_YEAR_SIGNAL] =
     g_signal_new (I_("next-year"),
@@ -815,7 +853,7 @@ gtk_calendar_init (GtkCalendar *calendar)
   /* Select current day */
   calendar->date = g_date_time_new_from_unix_local (0);
   now = g_date_time_new_now_local ();
-  gtk_calendar_select_day (calendar, now);
+  calendar_select_day_internal (calendar, now, FALSE);
   g_date_time_unref (now);
 
   /* We just initialized the year label, now add some space to it so
@@ -840,7 +878,7 @@ calendar_set_month_prev (GtkCalendar *calendar)
 
   new_date = g_date_time_add_months (calendar->date, -1);
 
-  gtk_calendar_select_day (calendar, new_date);
+  calendar_select_day_internal (calendar, new_date, FALSE);
   g_date_time_unref (new_date);
 
   g_signal_emit (calendar, gtk_calendar_signals[PREV_MONTH_SIGNAL], 0);
@@ -853,7 +891,7 @@ calendar_set_month_next (GtkCalendar *calendar)
 
   new_date = g_date_time_add_months (calendar->date, 1);
 
-  gtk_calendar_select_day (calendar, new_date);
+  calendar_select_day_internal (calendar, new_date, FALSE);
   g_date_time_unref (new_date);
 
   g_signal_emit (calendar, gtk_calendar_signals[NEXT_MONTH_SIGNAL], 0);
@@ -866,7 +904,7 @@ calendar_set_year_prev (GtkCalendar *calendar)
 
   new_date = g_date_time_add_years (calendar->date, -1);
 
-  gtk_calendar_select_day (calendar, new_date);
+  calendar_select_day_internal (calendar, new_date, FALSE);
   g_date_time_unref (new_date);
 
   g_signal_emit (calendar, gtk_calendar_signals[PREV_YEAR_SIGNAL], 0);
@@ -879,7 +917,7 @@ calendar_set_year_next (GtkCalendar *calendar)
 
   new_date = g_date_time_add_years (calendar->date, 1);
 
-  gtk_calendar_select_day (calendar, new_date);
+  calendar_select_day_internal (calendar, new_date, FALSE);
   g_date_time_unref (new_date);
 
   g_signal_emit (calendar, gtk_calendar_signals[NEXT_YEAR_SIGNAL], 0);
@@ -948,6 +986,214 @@ calendar_compute_days (GtkCalendar *calendar)
     }
 }
 
+static void
+calendar_update_day_labels (GtkCalendar *calendar)
+{
+  char buffer[255];
+  GDateTime *today;
+  int today_day;
+  int selected_year, selected_month, selected_day;
+  gboolean lower_limit_reached, upper_limit_reached;
+
+  today_day = -1;
+  today = g_date_time_new_now_local ();
+  if (g_date_time_get_year (calendar->date) == g_date_time_get_year (today))
+    if (g_date_time_get_month (calendar->date) == g_date_time_get_month (today))
+      today_day = g_date_time_get_day_of_month (today);
+  g_date_time_unref (today);
+
+  g_date_time_get_ymd (calendar->date, &selected_year, &selected_month, &selected_day);
+
+  lower_limit_reached = selected_year == YEAR_MIN && selected_month == G_DATE_JANUARY;
+  upper_limit_reached = selected_year == YEAR_MAX && selected_month == G_DATE_DECEMBER;
+
+  for (int row = 0; row < 6; row++)
+    for (int col = 0; col < 7; col++)
+      {
+        const int day = calendar->day[row][col];
+        GtkWidget *label = calendar->day_number_labels[row][col];
+        /* Translators: this defines whether the day numbers should use
+         * localized digits or the ones used in English (0123...).
+         *
+         * Translate to "%Id" if you want to use localized digits, or
+         * translate to "%d" otherwise.
+         *
+         * Note that translating this doesn't guarantee that you get localized
+         * digits. That needs support from your system and locale definition
+         * too.
+         */
+        g_snprintf (buffer, sizeof (buffer), C_ ("calendar:day:digits", "%d"), day);
+        gtk_label_set_label (GTK_LABEL (label), buffer);
+        gtk_widget_remove_css_class (label, "other-month");
+        gtk_widget_remove_css_class (label, "today");
+        gtk_widget_unset_state_flags (label, GTK_STATE_FLAG_CHECKED);
+        gtk_widget_unset_state_flags (label, GTK_STATE_FLAG_FOCUSED);
+        gtk_widget_unset_state_flags (label, GTK_STATE_FLAG_SELECTED);
+        gtk_widget_set_sensitive (label, TRUE);
+
+        if (calendar->focus_row == row && calendar->focus_col == col)
+          gtk_widget_set_state_flags (label, GTK_STATE_FLAG_FOCUSED, FALSE);
+
+        switch (calendar->day_month[row][col])
+          {
+          case MONTH_PREV:
+            gtk_widget_add_css_class (label, "other-month");
+            if (lower_limit_reached)
+              {
+                gtk_label_set_label (GTK_LABEL (label), NULL);
+                gtk_widget_set_sensitive (label, FALSE);
+              }
+            break;
+          case MONTH_CURRENT:
+            if (day == selected_day)
+              gtk_widget_set_state_flags (label, GTK_STATE_FLAG_SELECTED, FALSE);
+            if (day == today_day)
+              gtk_widget_add_css_class (label, "today");
+            if (calendar->marked_date[day - 1])
+              gtk_widget_set_state_flags (label, GTK_STATE_FLAG_CHECKED, FALSE);
+            break;
+          case MONTH_NEXT:
+            gtk_widget_add_css_class (label, "other-month");
+            if (upper_limit_reached)
+              {
+                gtk_label_set_label (GTK_LABEL (label), NULL);
+                gtk_widget_set_sensitive (label, FALSE);
+              }
+            break;
+          default:
+            g_assert_not_reached ();
+            break;
+          }
+      }
+}
+
+static void
+calendar_update_navigation_buttons (GtkCalendar *calendar)
+{
+  int year, month;
+  g_date_time_get_ymd (calendar->date, &year, &month, NULL);
+  for (int i = 0; i < 4; i++)
+    gtk_widget_set_sensitive (calendar->arrow_widgets[i], TRUE);
+  if (year == YEAR_MIN)
+    {
+      /* Cannot go back */
+      gtk_widget_set_sensitive (calendar->arrow_widgets[2], FALSE);
+      if (month == G_DATE_JANUARY)
+        gtk_widget_set_sensitive (calendar->arrow_widgets[0], FALSE);
+    }
+  else if (year == YEAR_MAX)
+    {
+      /* Cannot move forward */
+      gtk_widget_set_sensitive (calendar->arrow_widgets[3], FALSE);
+      if (month == G_DATE_DECEMBER)
+        gtk_widget_set_sensitive (calendar->arrow_widgets[1], FALSE);
+    }
+}
+
+static void
+calendar_select_day_internal (GtkCalendar *calendar,
+                              GDateTime   *date,
+                              gboolean     emit_day_signal)
+{
+  int new_month, new_year;
+  gboolean day_changed, month_changed, year_changed;
+  char buffer[255];
+  char *str;
+  time_t tmp_time;
+  struct tm *tm;
+  int i;
+
+  day_changed = g_date_time_get_day_of_month (calendar->date) != g_date_time_get_day_of_month (date);
+  month_changed = g_date_time_get_month (calendar->date) != g_date_time_get_month (date);
+  year_changed = g_date_time_get_year (calendar->date) != g_date_time_get_year (date);
+
+  if (!day_changed && !month_changed && !year_changed)
+    return;
+
+  new_year = g_date_time_get_year (date);
+  new_month = g_date_time_get_month (date);
+
+  g_date_time_unref (calendar->date);
+  calendar->date = g_date_time_ref (date);
+
+  tmp_time = 1;  /* Jan 1 1970, 00:00:01 UTC */
+  tm = gmtime (&tmp_time);
+  tm->tm_year = new_year - 1900;
+
+  /* Translators: This dictates how the year is displayed in
+   * gtkcalendar widget.  See strftime() manual for the format.
+   * Use only ASCII in the translation.
+   *
+   * "%Y" is appropriate for most locales.
+   */
+  strftime (buffer, sizeof (buffer), C_("calendar year format", "%Y"), tm);
+  str = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+  gtk_label_set_label (GTK_LABEL (calendar->year_label), str);
+  g_free (str);
+
+  /* Update month */
+
+  calendar_compute_days (calendar);
+  gtk_stack_set_visible_child_name (GTK_STACK (calendar->month_name_stack),
+                                    default_monthname[new_month - 1]);
+
+  calendar_update_navigation_buttons (calendar);
+
+  calendar_update_day_labels (calendar);
+
+  /* Update week number labels.
+   * We simply get the week number of calendar->date and add the others.
+   * simple. */
+  for (i = 0; i < 6; i ++)
+    {
+      int year = new_year;
+      int month, week;
+
+      month = new_month + calendar->day_month[i][6] - MONTH_CURRENT;
+
+      if (month < 1)
+        {
+          month += 12;
+          year -= 1;
+        }
+      else if (month > 12)
+        {
+          month -= 12;
+          year += 1;
+        }
+
+      week = week_of_year (year, month, calendar->day[i][6]);
+
+      /* Translators: this defines whether the week numbers should use
+       * localized digits or the ones used in English (0123...).
+       *
+       * Translate to "%Id" if you want to use localized digits, or
+       * translate to "%d" otherwise.
+       * Note that translating this doesn't guarantee that you get localized
+       * digits. That needs support from your system and locale definition
+       * too. */
+      g_snprintf (buffer, sizeof (buffer), C_("calendar:week:digits", "%d"), week);
+
+      gtk_label_set_label (GTK_LABEL (calendar->week_number_labels[i]), buffer);
+    }
+
+  if (day_changed)
+    {
+      g_object_notify (G_OBJECT (calendar), "day");
+
+      if (emit_day_signal)
+        g_signal_emit (calendar, gtk_calendar_signals[DAY_SELECTED_SIGNAL], 0);
+    }
+
+  if (month_changed)
+    g_object_notify (G_OBJECT (calendar), "month");
+
+  if (year_changed)
+    g_object_notify (G_OBJECT (calendar), "year");
+
+  g_object_notify (G_OBJECT (calendar), "date");
+}
+
 static void
 calendar_select_and_focus_day (GtkCalendar *calendar,
                                int          day)
@@ -973,7 +1219,7 @@ calendar_select_and_focus_day (GtkCalendar *calendar,
                                     day,
                                     0, 0, 0);
 
-  gtk_calendar_select_day (calendar, new_date);
+  calendar_select_day_internal (calendar, new_date, TRUE);
   g_date_time_unref (new_date);
 }
 
@@ -984,42 +1230,20 @@ gtk_calendar_set_property (GObject      *object,
                            GParamSpec   *pspec)
 {
   GtkCalendar *calendar = GTK_CALENDAR (object);
-  GDateTime *date;
 
   switch (prop_id)
     {
+    case PROP_DATE:
+      gtk_calendar_set_date (calendar, g_value_get_boxed (value));
+      break;
     case PROP_YEAR:
-      date = g_date_time_new_local (g_value_get_int (value),
-                                    g_date_time_get_month (calendar->date),
-                                    g_date_time_get_day_of_month (calendar->date),
-                                    0, 0, 0);
-      if (date)
-        {
-          gtk_calendar_select_day (calendar, date);
-          g_date_time_unref (date);
-        }
+      gtk_calendar_set_year (calendar, g_value_get_int (value));
       break;
     case PROP_MONTH:
-      date = g_date_time_new_local (g_date_time_get_year (calendar->date),
-                                    g_value_get_int (value) + 1,
-                                    g_date_time_get_day_of_month (calendar->date),
-                                    0, 0, 0);
-      if (date)
-        {
-          gtk_calendar_select_day (calendar, date);
-          g_date_time_unref (date);
-        }
+      gtk_calendar_set_month (calendar, g_value_get_int (value));
       break;
     case PROP_DAY:
-      date = g_date_time_new_local (g_date_time_get_year (calendar->date),
-                                    g_date_time_get_month (calendar->date),
-                                    g_value_get_int (value),
-                                    0, 0, 0);
-      if (date)
-        {
-          gtk_calendar_select_day (calendar, date);
-          g_date_time_unref (date);
-        }
+      gtk_calendar_set_day (calendar, g_value_get_int (value));
       break;
     case PROP_SHOW_HEADING:
       gtk_calendar_set_show_heading (calendar, g_value_get_boolean (value));
@@ -1046,14 +1270,17 @@ gtk_calendar_get_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_DATE:
+      g_value_take_boxed (value, gtk_calendar_get_date (calendar));
+      break;
     case PROP_YEAR:
-      g_value_set_int (value, g_date_time_get_year (calendar->date));
+      g_value_set_int (value, gtk_calendar_get_year (calendar));
       break;
     case PROP_MONTH:
-      g_value_set_int (value, g_date_time_get_month (calendar->date) - 1);
+      g_value_set_int (value, gtk_calendar_get_month (calendar));
       break;
     case PROP_DAY:
-      g_value_set_int (value, g_date_time_get_day_of_month (calendar->date));
+      g_value_set_int (value, gtk_calendar_get_day (calendar));
       break;
     case PROP_SHOW_HEADING:
       g_value_set_boolean (value, gtk_calendar_get_show_heading (calendar));
@@ -1159,66 +1386,73 @@ move_focus (GtkCalendar *calendar,
             int          updown)
 {
   GtkTextDirection text_dir = gtk_widget_get_direction (GTK_WIDGET (calendar));
-  int x, y;
+  int focus_row, focus_col;
+  GtkWidget *label;
+
+  focus_row = calendar->focus_row;
+  focus_col = calendar->focus_col;
 
   if (updown == 1)
     {
-      if (calendar->focus_row > 0)
-        calendar->focus_row--;
-      if (calendar->focus_row < 0)
-        calendar->focus_row = 5;
-      if (calendar->focus_col < 0)
-        calendar->focus_col = 6;
+      if (focus_row > 0)
+        focus_row--;
+      if (focus_row < 0)
+        focus_row = 5;
+      if (focus_col < 0)
+        focus_col = 6;
     }
   else if (updown == -1)
     {
-      if (calendar->focus_row < 5)
-        calendar->focus_row++;
-      if (calendar->focus_col < 0)
-        calendar->focus_col = 0;
+      if (focus_row < 5)
+        focus_row++;
+      if (focus_col < 0)
+        focus_col = 0;
     }
   else if ((text_dir == GTK_TEXT_DIR_LTR && direction == -1) ||
            (text_dir == GTK_TEXT_DIR_RTL && direction == 1))
     {
-      if (calendar->focus_col > 0)
-          calendar->focus_col--;
-      else if (calendar->focus_row > 0)
+      if (focus_col > 0)
+        focus_col--;
+      else if (focus_row > 0)
         {
-          calendar->focus_col = 6;
-          calendar->focus_row--;
+          focus_col = 6;
+          focus_row--;
         }
 
-      if (calendar->focus_col < 0)
-        calendar->focus_col = 6;
-      if (calendar->focus_row < 0)
-        calendar->focus_row = 5;
+      if (focus_col < 0)
+        focus_col = 6;
+      if (focus_row < 0)
+        focus_row = 5;
     }
   else
     {
-      if (calendar->focus_col < 6)
-        calendar->focus_col++;
-      else if (calendar->focus_row < 5)
+      if (focus_col < 6)
+        focus_col++;
+      else if (focus_row < 5)
         {
-          calendar->focus_col = 0;
-          calendar->focus_row++;
+          focus_col = 0;
+          focus_row++;
         }
 
-      if (calendar->focus_col < 0)
-        calendar->focus_col = 0;
-      if (calendar->focus_row < 0)
-        calendar->focus_row = 0;
+      if (focus_col < 0)
+        focus_col = 0;
+      if (focus_row < 0)
+        focus_row = 0;
     }
 
-  for (y = 0; y < 6; y ++)
-    for (x = 0; x < 7; x ++)
-      {
-        GtkWidget *label = calendar->day_number_labels[y][x];
+  if (!gtk_widget_is_sensitive (calendar->day_number_labels[focus_row][focus_col]))
+    return; /* Not sensitive, not reachable. */
 
-        if (calendar->focus_row == y && calendar->focus_col == x)
-          gtk_widget_set_state_flags (label, GTK_STATE_FLAG_FOCUSED, FALSE);
-        else
-          gtk_widget_unset_state_flags (label, GTK_STATE_FLAG_FOCUSED);
-      }
+  label = calendar->day_number_labels[calendar->focus_row][calendar->focus_col];
+  gtk_widget_unset_state_flags (label, GTK_STATE_FLAG_FOCUSED);
+  calendar_invalidate_day (calendar, calendar->focus_row, calendar->focus_col);
+
+  calendar->focus_row = focus_row;
+  calendar->focus_col = focus_col;
+
+  label = calendar->day_number_labels[focus_row][focus_col];
+  gtk_widget_set_state_flags (label, GTK_STATE_FLAG_FOCUSED, FALSE);
+  calendar_invalidate_day (calendar, focus_row, focus_col);
 }
 
 static gboolean
@@ -1230,64 +1464,60 @@ gtk_calendar_key_controller_key_pressed (GtkEventControllerKey *controller,
 {
   GtkCalendar *calendar = GTK_CALENDAR (widget);
   int return_val;
-  int old_focus_row;
-  int old_focus_col;
   int row, col, day;
+#ifdef __APPLE__
+  GdkModifierType modifier = GDK_META_MASK;
+#else
+  GdkModifierType modifier = GDK_CONTROL_MASK;
+#endif
 
   return_val = FALSE;
 
-  old_focus_row = calendar->focus_row;
-  old_focus_col = calendar->focus_col;
-
   switch (keyval)
     {
     case GDK_KEY_KP_Left:
     case GDK_KEY_Left:
       return_val = TRUE;
-      if (state & GDK_CONTROL_MASK)
-        calendar_set_month_prev (calendar);
+      if (state & modifier)
+        if (!gtk_widget_is_sensitive (calendar->arrow_widgets[0]))
+          break; /* Not allowed with the mouse, not allowed with the keyboard. */
+        else
+          calendar_set_month_prev (calendar);
       else
-        {
-          move_focus (calendar, -1, 0);
-          calendar_invalidate_day (calendar, old_focus_row, old_focus_col);
-          calendar_invalidate_day (calendar, calendar->focus_row, calendar->focus_col);
-        }
+        move_focus (calendar, -1, 0);
       break;
     case GDK_KEY_KP_Right:
     case GDK_KEY_Right:
       return_val = TRUE;
-      if (state & GDK_CONTROL_MASK)
-        calendar_set_month_next (calendar);
+      if (state & modifier)
+        if (!gtk_widget_is_sensitive (calendar->arrow_widgets[1]))
+          break; /* Not allowed with the mouse, not allowed with the keyboard. */
+        else
+          calendar_set_month_next (calendar);
       else
-        {
-          move_focus (calendar, 1, 0);
-          calendar_invalidate_day (calendar, old_focus_row, old_focus_col);
-          calendar_invalidate_day (calendar, calendar->focus_row, calendar->focus_col);
-        }
+        move_focus (calendar, 1, 0);
       break;
     case GDK_KEY_KP_Up:
     case GDK_KEY_Up:
       return_val = TRUE;
-      if (state & GDK_CONTROL_MASK)
-        calendar_set_year_prev (calendar);
+      if (state & modifier)
+        if (!gtk_widget_is_sensitive (calendar->arrow_widgets[2]))
+          break; /* Not allowed with the mouse, not allowed with the keyboard. */
+        else
+          calendar_set_year_prev (calendar);
       else
-        {
-          move_focus (calendar, 0, 1);
-          calendar_invalidate_day (calendar, old_focus_row, old_focus_col);
-          calendar_invalidate_day (calendar, calendar->focus_row, calendar->focus_col);
-        }
+        move_focus (calendar, 0, 1);
       break;
     case GDK_KEY_KP_Down:
     case GDK_KEY_Down:
       return_val = TRUE;
-      if (state & GDK_CONTROL_MASK)
-        calendar_set_year_next (calendar);
+      if (state & modifier)
+        if (!gtk_widget_is_sensitive (calendar->arrow_widgets[3]))
+          break; /* Not allowed with the mouse, not allowed with the keyboard. */
+        else
+          calendar_set_year_next (calendar);
       else
-        {
-          move_focus (calendar, 0, -1);
-          calendar_invalidate_day (calendar, old_focus_row, old_focus_col);
-          calendar_invalidate_day (calendar, calendar->focus_row, calendar->focus_col);
-        }
+        move_focus (calendar, 0, -1);
       break;
     case GDK_KEY_KP_Space:
     case GDK_KEY_space:
@@ -1347,168 +1577,17 @@ gtk_calendar_new (void)
  * @date: (transfer none): a `GDateTime` representing the day to select
  *
  * Switches to @date's year and month and select its day.
+ *
+ * Deprecated: 4.20: Use [method@Calendar.set_date] instead.
  */
 void
 gtk_calendar_select_day (GtkCalendar *calendar,
                          GDateTime   *date)
 {
-  GDateTime *today;
-  int new_day, new_month, new_year;
-  gboolean day_changed, month_changed, year_changed;
-  char buffer[255];
-  char *str;
-  time_t tmp_time;
-  struct tm *tm;
-  int i;
-  int x, y;
-  int today_day;
-
   g_return_if_fail (GTK_IS_CALENDAR (calendar));
   g_return_if_fail (date != NULL);
 
-  day_changed = g_date_time_get_day_of_month (calendar->date) != g_date_time_get_day_of_month (date);
-  month_changed = g_date_time_get_month (calendar->date) != g_date_time_get_month (date);
-  year_changed = g_date_time_get_year (calendar->date) != g_date_time_get_year (date);
-
-  if (!day_changed && !month_changed && !year_changed)
-    return;
-
-  new_year = g_date_time_get_year (date);
-  new_month = g_date_time_get_month (date);
-  new_day = g_date_time_get_day_of_month (date);
-
-  g_date_time_unref (calendar->date);
-  calendar->date = g_date_time_ref (date);
-
-  tmp_time = 1;  /* Jan 1 1970, 00:00:01 UTC */
-  tm = gmtime (&tmp_time);
-  tm->tm_year = new_year - 1900;
-
-  /* Translators: This dictates how the year is displayed in
-   * gtkcalendar widget.  See strftime() manual for the format.
-   * Use only ASCII in the translation.
-   *
-   * "%Y" is appropriate for most locales.
-   */
-  strftime (buffer, sizeof (buffer), C_("calendar year format", "%Y"), tm);
-  str = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
-  gtk_label_set_label (GTK_LABEL (calendar->year_label), str);
-  g_free (str);
-
-  /* Update month */
-
-  calendar_compute_days (calendar);
-  gtk_stack_set_visible_child_name (GTK_STACK (calendar->month_name_stack),
-                                    default_monthname[new_month - 1]);
-
-  today = g_date_time_new_now_local ();
-
-  if (g_date_time_get_year (calendar->date) == g_date_time_get_year (today) &&
-      g_date_time_get_month (calendar->date) == g_date_time_get_month (today))
-    today_day = g_date_time_get_day_of_month (today);
-  else
-    today_day = -1;
-
-  g_date_time_unref (today);
-
-  /* Update day labels */
-  for (y = 0; y < 6; y ++)
-    for (x = 0; x < 7; x ++)
-      {
-        const int day = calendar->day[y][x];
-        GtkWidget *label = calendar->day_number_labels[y][x];
-        /* Translators: this defines whether the day numbers should use
-         * localized digits or the ones used in English (0123...).
-         *
-         * Translate to "%Id" if you want to use localized digits, or
-         * translate to "%d" otherwise.
-         *
-         * Note that translating this doesn't guarantee that you get localized
-         * digits. That needs support from your system and locale definition
-         * too.
-         */
-        g_snprintf (buffer, sizeof (buffer), C_("calendar:day:digits", "%d"), day);
-
-        gtk_label_set_label (GTK_LABEL (label), buffer);
-
-        if (calendar->day_month[y][x] == MONTH_PREV ||
-            calendar->day_month[y][x] == MONTH_NEXT)
-          gtk_widget_add_css_class (label, "other-month");
-        else
-          gtk_widget_remove_css_class (label, "other-month");
-
-        if (calendar->marked_date[day-1] &&
-            calendar->day_month[y][x] == MONTH_CURRENT)
-          gtk_widget_set_state_flags (label, GTK_STATE_FLAG_CHECKED, FALSE);
-        else
-          gtk_widget_unset_state_flags (label, GTK_STATE_FLAG_CHECKED);
-
-        if (new_day == day &&
-            calendar->day_month[y][x] == MONTH_CURRENT)
-          gtk_widget_set_state_flags (label, GTK_STATE_FLAG_SELECTED, FALSE);
-        else
-          gtk_widget_unset_state_flags (label, GTK_STATE_FLAG_SELECTED);
-
-        if (calendar->focus_row == y && calendar->focus_col == x)
-          gtk_widget_set_state_flags (label, GTK_STATE_FLAG_FOCUSED, FALSE);
-        else
-          gtk_widget_unset_state_flags (label, GTK_STATE_FLAG_FOCUSED);
-
-        if (day == today_day &&
-            calendar->day_month[y][x] == MONTH_CURRENT)
-          gtk_widget_add_css_class (label, "today");
-        else
-          gtk_widget_remove_css_class (label, "today");
-      }
-
-  /* Update week number labels.
-   * We simply get the week number of calendar->date and add the others.
-   * simple. */
-  for (i = 0; i < 6; i ++)
-    {
-      int year = new_year;
-      int month, week;
-
-      month = new_month + calendar->day_month[i][6] - MONTH_CURRENT;
-
-      if (month < 1)
-        {
-          month += 12;
-          year -= 1;
-        }
-      else if (month > 12)
-        {
-          month -= 12;
-          year += 1;
-        }
-
-      week = week_of_year (year, month, calendar->day[i][6]);
-
-      /* Translators: this defines whether the week numbers should use
-       * localized digits or the ones used in English (0123...).
-       *
-       * Translate to "%Id" if you want to use localized digits, or
-       * translate to "%d" otherwise.
-       * Note that translating this doesn't guarantee that you get localized
-       * digits. That needs support from your system and locale definition
-       * too. */
-      g_snprintf (buffer, sizeof (buffer), C_("calendar:week:digits", "%d"), week);
-
-      gtk_label_set_label (GTK_LABEL (calendar->week_number_labels[i]), buffer);
-    }
-
-  if (day_changed)
-    {
-      g_object_notify (G_OBJECT (calendar), "day");
-      g_signal_emit (calendar, gtk_calendar_signals[DAY_SELECTED_SIGNAL], 0);
-    }
-
-  if (month_changed)
-    g_object_notify (G_OBJECT (calendar), "month");
-
-  if (year_changed)
-    g_object_notify (G_OBJECT (calendar), "year");
-
+  calendar_select_day_internal (calendar, date, TRUE);
 }
 
 /**
@@ -1627,6 +1706,25 @@ gtk_calendar_unmark_day (GtkCalendar *calendar,
     }
 }
 
+/**
+ * gtk_calendar_set_date:
+ * @self: a `GtkCalendar`.
+ * @date: (transfer none): a `GDateTime` representing the day to select
+ *
+ * Switches to @date's year and month and selects its day.
+ *
+ * Since: 4.20
+ */
+void
+gtk_calendar_set_date (GtkCalendar *self,
+                       GDateTime   *date)
+{
+  g_return_if_fail (GTK_IS_CALENDAR (self));
+  g_return_if_fail (date != NULL);
+
+  calendar_select_day_internal (self, date, TRUE);
+}
+
 /**
  * gtk_calendar_get_date:
  * @self: a `GtkCalendar`
@@ -1636,7 +1734,7 @@ gtk_calendar_unmark_day (GtkCalendar *calendar,
  *
  * The returned date is in the local time zone.
  *
- * Returns: (transfer full): the `GDate` representing the shown date
+ * Returns: (transfer full): the `GDateTime` representing the selected date
  */
 GDateTime *
 gtk_calendar_get_date (GtkCalendar *self)
@@ -1647,9 +1745,9 @@ gtk_calendar_get_date (GtkCalendar *self)
 }
 
 /**
- * gtk_calendar_set_show_week_numbers: (attributes org.gtk.Method.set_property=show-week-numbers)
+ * gtk_calendar_set_show_week_numbers:
  * @self: a `GtkCalendar`
- * @value: whether to show week numbers on the left of the days
+ * @value: whether to show week numbers alongside the days
  *
  * Sets whether week numbers are shown in the calendar.
  */
@@ -1673,7 +1771,7 @@ gtk_calendar_set_show_week_numbers (GtkCalendar *self,
 }
 
 /**
- * gtk_calendar_get_show_week_numbers: (attributes org.gtk.Method.get_property=show-week-numbers)
+ * gtk_calendar_get_show_week_numbers:
  * @self: a `GtkCalendar`
  *
  * Returns whether @self is showing week numbers right
@@ -1693,7 +1791,7 @@ gtk_calendar_get_show_week_numbers (GtkCalendar *self)
 }
 
 /**
- * gtk_calendar_set_show_heading: (attributes org.gtk.Method.set_property=show-heading)
+ * gtk_calendar_set_show_heading:
  * @self: a `GtkCalendar`
  * @value: Whether to show the heading in the calendar
  *
@@ -1719,7 +1817,7 @@ gtk_calendar_set_show_heading (GtkCalendar *self,
 }
 
 /**
- * gtk_calendar_get_show_heading: (attributes org.gtk.Method.get_property=show-heading)
+ * gtk_calendar_get_show_heading:
  * @self: a `GtkCalendar`
  *
  * Returns whether @self is currently showing the heading.
@@ -1738,7 +1836,7 @@ gtk_calendar_get_show_heading (GtkCalendar *self)
 }
 
 /**
- * gtk_calendar_set_show_day_names: (attributes org.gtk.Method.set_property=show-day-names)
+ * gtk_calendar_set_show_day_names:
  * @self: a `GtkCalendar`
  * @value: Whether to show day names above the day numbers
  *
@@ -1764,7 +1862,7 @@ gtk_calendar_set_show_day_names (GtkCalendar *self,
 }
 
 /**
- * gtk_calendar_get_show_day_names: (attributes org.gtk.Method.get_property=show-day-names)
+ * gtk_calendar_get_show_day_names:
  * @self: a `GtkCalendar`
  *
  * Returns whether @self is currently showing the names
@@ -1782,3 +1880,166 @@ gtk_calendar_get_show_day_names (GtkCalendar *self)
 
   return self->show_day_names;
 }
+
+/**
+ * gtk_calendar_set_day:
+ * @self: a `GtkCalendar`
+ * @day: The desired day for the selected date (as a number between 1 and 31).
+ *
+ * Sets the day for the selected date.
+ *
+ * The new date must be valid. For example, setting the day to 31 when the
+ * month is February will fail.
+ *
+ * Since: 4.14
+ */
+void
+gtk_calendar_set_day (GtkCalendar *self,
+                      int          day)
+{
+  GDateTime *date;
+
+  g_return_if_fail (GTK_IS_CALENDAR (self));
+  g_return_if_fail (day >= 1 && day <= 31);
+
+  if (day == g_date_time_get_day_of_month (self->date))
+    return;
+
+  date = g_date_time_new_local (g_date_time_get_year (self->date),
+                                g_date_time_get_month (self->date),
+                                day,
+                                0, 0, 0.0);
+  g_return_if_fail (date != NULL);
+
+  calendar_select_day_internal (self, date, TRUE);
+  g_date_time_unref (date);
+
+  g_object_notify (G_OBJECT (self), "day");
+}
+
+/**
+ * gtk_calendar_get_day:
+ * @self: a `GtkCalendar`
+ *
+ * Gets the day of the selected date.
+ *
+ * Returns: the day of the selected date.
+ *
+ * Since: 4.14
+ */
+int
+gtk_calendar_get_day (GtkCalendar *self)
+{
+  g_return_val_if_fail (GTK_IS_CALENDAR (self), -1);
+
+  return g_date_time_get_day_of_month (self->date);
+}
+
+/**
+ * gtk_calendar_set_month:
+ * @self: a `GtkCalendar`
+ * @month: The desired month for the selected date (as a number between 0 and 11).
+ *
+ * Sets the month for the selected date.
+ *
+ * The new date must be valid. For example, setting the month to 1 (February)
+ * when the day is 31 will fail.
+ *
+ * Since: 4.14
+ */
+void
+gtk_calendar_set_month (GtkCalendar *self,
+                        int          month)
+{
+  GDateTime *date;
+
+  g_return_if_fail (GTK_IS_CALENDAR (self));
+  g_return_if_fail (month >= 0 && month <= 11);
+
+  if (month == g_date_time_get_month (self->date) - 1)
+    return;
+
+  date = g_date_time_new_local (g_date_time_get_year (self->date),
+                                month + 1,
+                                g_date_time_get_day_of_month (self->date),
+                                0, 0, 0.0);
+  g_return_if_fail (date != NULL);
+
+  calendar_select_day_internal (self, date, TRUE);
+  g_date_time_unref (date);
+
+  g_object_notify (G_OBJECT (self), "month");
+}
+
+/**
+ * gtk_calendar_get_month:
+ * @self: a `GtkCalendar`
+ *
+ * Gets the month of the selected date.
+ *
+ * Returns: The month of the selected date (as a number between 0 and 11).
+ *
+ * Since: 4.14
+ */
+int
+gtk_calendar_get_month (GtkCalendar *self)
+{
+  g_return_val_if_fail (GTK_IS_CALENDAR (self), -1);
+
+  return g_date_time_get_month (self->date) - 1;
+}
+
+/**
+ * gtk_calendar_set_year:
+ * @self: a `GtkCalendar`
+ * @year: The desired year for the selected date (within [struct@GLib.DateTime]
+ *   limits, i.e. from 0001 to 9999).
+ *
+ * Sets the year for the selected date.
+ *
+ * The new date must be valid. For example, setting the year to 2023 when the
+ * date is February 29 will fail.
+ *
+ * Since: 4.14
+ */
+void
+gtk_calendar_set_year (GtkCalendar *self,
+                       int          year)
+{
+  GDateTime *date;
+
+  g_return_if_fail (GTK_IS_CALENDAR (self));
+  g_return_if_fail (year >= 1 && year <= 9999);
+
+  if (year == g_date_time_get_year (self->date))
+    return;
+
+  date = g_date_time_new_local (year,
+                                g_date_time_get_month (self->date),
+                                g_date_time_get_day_of_month (self->date),
+                                0, 0, 0.0);
+  g_return_if_fail (date != NULL);
+
+  calendar_select_day_internal (self, date, TRUE);
+  g_date_time_unref (date);
+
+  g_object_notify (G_OBJECT (self), "year");
+}
+
+/**
+ * gtk_calendar_get_year:
+ * @self: a `GtkCalendar`
+ *
+ * Gets the year of the selected date.
+ *
+ * Returns: the year of the selected date.
+ *
+ * Since: 4.14
+ */
+int
+gtk_calendar_get_year (GtkCalendar *self)
+{
+  g_return_val_if_fail (GTK_IS_CALENDAR (self), -1);
+
+  return g_date_time_get_year (self->date);
+}
