@@ -19,13 +19,13 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
 /**
  * GtkEditable:
  *
- * `GtkEditable` is an interface for text editing widgets.
+ * Interface for single-line text editing widgets.
  *
  * Typical examples of editable widgets are [class@Gtk.Entry] and
  * [class@Gtk.SpinButton]. It contains functions for generically manipulating
@@ -161,7 +161,7 @@
 #include "gtkeditable.h"
 #include "gtkentrybuffer.h"
 #include "gtkmarshalers.h"
-#include "gtkintl.h"
+#include "gdk/gdkmarshalers.h"
 #include "gtkprivate.h"
 
 G_DEFINE_INTERFACE (GtkEditable, gtk_editable, GTK_TYPE_WIDGET)
@@ -343,13 +343,13 @@ gtk_editable_default_init (GtkEditableInterface *iface)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkEditableInterface, delete_text),
                   NULL, NULL,
-                  _gtk_marshal_VOID__INT_INT,
+                  _gdk_marshal_VOID__INT_INT,
                   G_TYPE_NONE, 2,
                   G_TYPE_INT,
                   G_TYPE_INT);
   g_signal_set_va_marshaller (signals[DELETE_TEXT],
                               G_TYPE_FROM_INTERFACE (iface),
-                              _gtk_marshal_VOID__INT_INTv);
+                              _gdk_marshal_VOID__INT_INTv);
 
   /**
    * GtkEditable::changed:
@@ -374,39 +374,33 @@ gtk_editable_default_init (GtkEditableInterface *iface)
                   G_TYPE_NONE, 0);
 
   /**
-   * GtkEditable:text: (attributes org.gtk.Property.get=gtk_editable_get_text org.gtk.Property.set=gtk_editable_set_text)
+   * GtkEditable:text:
    *
    * The contents of the entry.
    */
   g_object_interface_install_property (iface,
-      g_param_spec_string ("text",
-                           P_("Text"),
-                           P_("The contents of the entry"),
+      g_param_spec_string ("text", NULL, NULL,
                            "",
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkEditable:cursor-position: (attributes org.gtk.Property.get=gtk_editable_get_position org.gtk.Property.set=gtk_editable_set_position)
+   * GtkEditable:cursor-position: (getter get_position) (setter set_position)
    *
    * The current position of the insertion cursor in chars.
    */
   g_object_interface_install_property (iface,
-      g_param_spec_int ("cursor-position",
-                        P_("Cursor Position"),
-                        P_("The current position of the insertion cursor in chars"),
+      g_param_spec_int ("cursor-position", NULL, NULL,
                         0, GTK_ENTRY_BUFFER_MAX_SIZE,
                         0,
                         GTK_PARAM_READABLE));
 
   /**
-   * GtkEditable:enable-undo: (attributes org.gtk.Property.get=gtk_editable_get_enable_undo org.gtk.Property.setg=gtk_editable_set_enable_undo)
+   * GtkEditable:enable-undo:
    *
    * If undo/redo should be enabled for the editable.
    */
   g_object_interface_install_property (iface,
-      g_param_spec_boolean ("enable-undo",
-                            P_("Enable Undo"),
-                            P_("If undo/redo should be enabled for the editable"),
+      g_param_spec_boolean ("enable-undo", NULL, NULL,
                             TRUE,
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
@@ -416,62 +410,52 @@ gtk_editable_default_init (GtkEditableInterface *iface)
    * The position of the opposite end of the selection from the cursor in chars.
    */
   g_object_interface_install_property (iface,
-      g_param_spec_int ("selection-bound",
-                        P_("Selection Bound"),
-                        P_("The position of the opposite end of the selection from the cursor in chars"),
+      g_param_spec_int ("selection-bound", NULL, NULL,
                         0, GTK_ENTRY_BUFFER_MAX_SIZE,
                         0,
                         GTK_PARAM_READABLE));
 
   /**
-   * GtkEditable:editable: (attributes org.gtk.Property.get=gtk_editable_get_editable org.gtk.Property.set=gtk_editable_set_editable)
+   * GtkEditable:editable:
    *
    * Whether the entry contents can be edited.
    */
   g_object_interface_install_property (iface,
-      g_param_spec_boolean ("editable",
-                            P_("Editable"),
-                            P_("Whether the entry contents can be edited"),
+      g_param_spec_boolean ("editable", NULL, NULL,
                             TRUE,
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkEditable:width-chars: (attributes org.gtk.Property.get=gtk_editable_get_width_chars org.gtk.Property.set=gtk_editable_set_width_chars)
+   * GtkEditable:width-chars:
    *
    * Number of characters to leave space for in the entry.
    */
   g_object_interface_install_property (iface,
-      g_param_spec_int ("width-chars",
-                        P_("Width in chars"),
-                        P_("Number of characters to leave space for in the entry"),
+      g_param_spec_int ("width-chars", NULL, NULL,
                         -1, G_MAXINT,
                         -1,
                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkEditable:max-width-chars: (attributes org.gtk.Property.get=gtk_editable_get_max_width_chars org.gtk.Property.set=gtk_editable_set_max_width_chars)
+   * GtkEditable:max-width-chars:
    *
    * The desired maximum width of the entry, in characters.
    */
   g_object_interface_install_property (iface,
-      g_param_spec_int ("max-width-chars",
-                        P_("Maximum width in characters"),
-                        P_("The desired maximum width of the entry, in characters"),
+      g_param_spec_int ("max-width-chars", NULL, NULL,
                         -1, G_MAXINT,
                         -1,
                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   /**
-   * GtkEditable:xalign: (attributes org.gtk.Property.get=gtk_editable_get_alignment org.gtk.Property.set=gtk_editable_set_alignment)
+   * GtkEditable:xalign: (getter get_alignment) (setter set_alignment)
    *
    * The horizontal alignment, from 0 (left) to 1 (right).
    *
    * Reversed for RTL layouts.
    */
   g_object_interface_install_property (iface,
-      g_param_spec_float ("xalign",
-                          P_("X align"),
-                          P_("The horizontal alignment, from 0 (left) to 1 (right). Reversed for RTL layouts."),
+      g_param_spec_float ("xalign", NULL, NULL,
                           0.0, 1.0,
                           0.0,
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
@@ -480,7 +464,7 @@ gtk_editable_default_init (GtkEditableInterface *iface)
 /**
  * gtk_editable_insert_text: (virtual do_insert_text)
  * @editable: a `GtkEditable`
- * @text: the text to append
+ * @text: the text to insert
  * @length: the length of the text in bytes, or -1
  * @position: (inout): location of the position text will be inserted at
  *
@@ -498,11 +482,16 @@ gtk_editable_insert_text (GtkEditable *editable,
                           int         *position)
 {
   g_return_if_fail (GTK_IS_EDITABLE (editable));
+  g_return_if_fail (text != NULL);
+  g_return_if_fail (length >= -1);
   g_return_if_fail (position != NULL);
 
+  if (text == NULL)
+    text = "";
+
   if (length < 0)
     length = strlen (text);
-  
+
   GTK_EDITABLE_GET_IFACE (editable)->do_insert_text (editable, text, length, position);
 }
 
@@ -527,6 +516,8 @@ gtk_editable_delete_text (GtkEditable *editable,
                           int          end_pos)
 {
   g_return_if_fail (GTK_IS_EDITABLE (editable));
+  g_return_if_fail (start_pos >= 0);
+  g_return_if_fail (end_pos == -1 || end_pos >= start_pos);
 
   GTK_EDITABLE_GET_IFACE (editable)->do_delete_text (editable, start_pos, end_pos);
 }
@@ -560,6 +551,8 @@ gtk_editable_get_chars (GtkEditable *editable,
   int start_index,end_index;
 
   g_return_val_if_fail (GTK_IS_EDITABLE (editable), NULL);
+  g_return_val_if_fail (start_pos >= 0, NULL);
+  g_return_val_if_fail (end_pos == -1 || end_pos >= start_pos, NULL);
 
   text = GTK_EDITABLE_GET_IFACE (editable)->get_text (editable);
   length = g_utf8_strlen (text, -1);
@@ -577,7 +570,7 @@ gtk_editable_get_chars (GtkEditable *editable,
 }
 
 /**
- * gtk_editable_get_text: (attributes org.gtk.Method.get_property=text)
+ * gtk_editable_get_text:
  * @editable: a `GtkEditable`
  *
  * Retrieves the contents of @editable.
@@ -595,7 +588,7 @@ gtk_editable_get_text (GtkEditable *editable)
 }
 
 /**
- * gtk_editable_set_text: (attributes org.gtk.Method.set_property=text)
+ * gtk_editable_set_text:
  * @editable: a `GtkEditable`
  * @text: the text to set
  *
@@ -610,6 +603,7 @@ gtk_editable_set_text (GtkEditable *editable,
   int pos;
 
   g_return_if_fail (GTK_IS_EDITABLE (editable));
+  g_return_if_fail (text != NULL);
 
   g_object_freeze_notify (G_OBJECT (editable));
   gtk_editable_delete_text (editable, 0, -1);
@@ -619,7 +613,7 @@ gtk_editable_set_text (GtkEditable *editable,
 }
 
 /**
- * gtk_editable_set_position: (attributes org.gtk.Method.set_property=cursor-position)
+ * gtk_editable_set_position: (set-property cursor-position)
  * @editable: a `GtkEditable`
  * @position: the position of the cursor
  *
@@ -641,7 +635,7 @@ gtk_editable_set_position (GtkEditable *editable,
 }
 
 /**
- * gtk_editable_get_position: (attributes org.gtk.Method.get_property=cursor-position)
+ * gtk_editable_get_position: (get-property cursor-position)
  * @editable: a `GtkEditable`
  *
  * Retrieves the current position of the cursor relative
@@ -744,7 +738,7 @@ gtk_editable_select_region (GtkEditable *editable,
 }
 
 /**
- * gtk_editable_set_editable: (attributes org.gtk.Method.set_property=editable)
+ * gtk_editable_set_editable:
  * @editable: a `GtkEditable`
  * @is_editable: %TRUE if the user is allowed to edit the text
  *   in the widget
@@ -761,7 +755,7 @@ gtk_editable_set_editable (GtkEditable *editable,
 }
 
 /**
- * gtk_editable_get_editable: (attributes org.gtk.Method.get_property=editable)
+ * gtk_editable_get_editable:
  * @editable: a `GtkEditable`
  *
  * Retrieves whether @editable is editable.
@@ -782,7 +776,7 @@ gtk_editable_get_editable (GtkEditable *editable)
 
 
 /**
- * gtk_editable_get_alignment: (attributes org.gtk.Method.get_property=xalign)
+ * gtk_editable_get_alignment: (get-property xalign)
  * @editable: a `GtkEditable`
  *
  * Gets the alignment of the editable.
@@ -802,7 +796,7 @@ gtk_editable_get_alignment (GtkEditable *editable)
 }
 
 /**
- * gtk_editable_set_alignment: (attributes org.gtk.Method.set_property=xalign)
+ * gtk_editable_set_alignment: (set-property xalign)
  * @editable: a `GtkEditable`
  * @xalign: The horizontal alignment, from 0 (left) to 1 (right).
  *   Reversed for RTL layouts
@@ -822,7 +816,7 @@ gtk_editable_set_alignment (GtkEditable *editable,
 }
 
 /**
- * gtk_editable_get_width_chars: (attributes org.gtk.Method.get_property=width-chars)
+ * gtk_editable_get_width_chars:
  * @editable: a `GtkEditable`
  *
  * Gets the number of characters of space reserved
@@ -843,7 +837,7 @@ gtk_editable_get_width_chars (GtkEditable *editable)
 }
 
 /**
- * gtk_editable_set_width_chars: (attributes org.gtk.Method.set_property=width-chars)
+ * gtk_editable_set_width_chars:
  * @editable: a `GtkEditable`
  * @n_chars: width in chars
  *
@@ -864,7 +858,7 @@ gtk_editable_set_width_chars (GtkEditable *editable,
 }
 
 /**
- * gtk_editable_get_max_width_chars: (attributes org.gtk.Method.get_property=max-width-chars)
+ * gtk_editable_get_max_width_chars:
  * @editable: a `GtkEditable`
  *
  * Retrieves the desired maximum width of @editable, in characters.
@@ -884,7 +878,7 @@ gtk_editable_get_max_width_chars (GtkEditable *editable)
 }
 
 /**
- * gtk_editable_set_max_width_chars: (attributes org.gtk.Method.set_property=max-width-chars)
+ * gtk_editable_set_max_width_chars:
  * @editable: a `GtkEditable`
  * @n_chars: the new desired maximum width, in characters
  *
@@ -900,7 +894,7 @@ gtk_editable_set_max_width_chars (GtkEditable *editable,
 }
 
 /**
- * gtk_editable_get_enable_undo: (attributes org.gtk.Method.get_property=enable-undo)
+ * gtk_editable_get_enable_undo:
  * @editable: a `GtkEditable`
  *
  * Gets if undo/redo actions are enabled for @editable
@@ -920,7 +914,7 @@ gtk_editable_get_enable_undo (GtkEditable *editable)
 }
 
 /**
- * gtk_editable_set_enable_undo: (attributes org.gtk.Method.set_property=enable-undo)
+ * gtk_editable_set_enable_undo:
  * @editable: a `GtkEditable`
  * @enable_undo: if undo/redo should be enabled
  *
@@ -1198,3 +1192,65 @@ gtk_editable_delegate_get_property (GObject    *object,
 
   return TRUE;
 }
+
+/**
+ * gtk_editable_delegate_get_accessible_platform_state:
+ * @editable: a `GtkEditable` implementation
+ * @state: what kind of accessible state to retrieve
+ *
+ * Retrieves the accessible platform state from the editable delegate.
+ *
+ * This is an helper function to retrieve the accessible state for
+ * `GtkEditable` interface implementations using a delegate pattern.
+ *
+ * You should call this function in your editable widget implementation
+ * of the [vfunc@Gtk.Accessible.get_platform_state] virtual function, for
+ * instance:
+ *
+ * ```c
+ * static void
+ * accessible_interface_init (GtkAccessibleInterface *iface)
+ * {
+ *   iface->get_platform_state = your_editable_get_accessible_platform_state;
+ * }
+ *
+ * static gboolean
+ * your_editable_get_accessible_platform_state (GtkAccessible *accessible,
+ *                                              GtkAccessiblePlatformState state)
+ * {
+ *   return gtk_editable_delegate_get_accessible_platform_state (GTK_EDITABLE (accessible), state);
+ * }
+ * ```
+ *
+ * Note that the widget which is the delegate *must* be a direct child of
+ * this widget, otherwise your implementation of [vfunc@Gtk.Accessible.get_platform_state]
+ * might not even be called, as the platform change will originate from
+ * the parent of the delegate, and, as a result, will not work properly.
+ *
+ * So, if you can't ensure the direct child condition, you should give the
+ * delegate the %GTK_ACCESSIBLE_ROLE_TEXT_BOX role, or you can
+ * change your tree to allow this function to work.
+ *
+ * Returns: the accessible platform state of the delegate
+ *
+ * Since: 4.10
+ */
+gboolean
+gtk_editable_delegate_get_accessible_platform_state (GtkEditable                *editable,
+                                                     GtkAccessiblePlatformState  state)
+{
+  GtkWidget *delegate = GTK_WIDGET (gtk_editable_get_delegate (editable));
+
+  switch (state)
+    {
+    case GTK_ACCESSIBLE_PLATFORM_STATE_FOCUSABLE:
+      return gtk_widget_get_focusable (delegate);
+    case GTK_ACCESSIBLE_PLATFORM_STATE_FOCUSED:
+      return gtk_widget_has_focus (delegate);
+    case GTK_ACCESSIBLE_PLATFORM_STATE_ACTIVE:
+      return FALSE;
+    default:
+      g_assert_not_reached ();
+      return FALSE;
+    }
+}
