@@ -257,7 +257,7 @@ gtk_editable_label_drag_accept (GtkDropTarget    *dest,
   if (!gtk_editable_get_editable (GTK_EDITABLE (self)))
     return FALSE;
 
-  if ((gdk_drop_get_actions (drop) & gtk_drop_target_get_actions (dest)) == 0)
+  if ((gdk_drop_get_actions (drop) & gtk_drop_target_get_actions (dest)) == GDK_ACTION_NONE)
     return FALSE;
 
   return gdk_content_formats_match (gtk_drop_target_get_formats (dest), gdk_drop_get_formats (drop));
