@@ -19,12 +19,15 @@
 
 #include "config.h"
 #include "gtkfilechooserutils.h"
-#include "gtkfilechooser.h"
+#include "deprecated/gtkfilechooser.h"
 #include "gtktypebuiltins.h"
 #include "gtkprivate.h"
+#include "gdktextureutilsprivate.h"
 #include <glib/gi18n-lib.h>
 
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 static gboolean       delegate_set_current_folder     (GtkFileChooser    *chooser,
 						       GFile             *file,
 						       GError           **error);
@@ -456,19 +459,21 @@ _gtk_file_info_get_icon (GFileInfo    *info,
                          GtkIconTheme *icon_theme)
 {
   GIcon *icon;
-  GdkPixbuf *pixbuf;
   const char *thumbnail_path;
 
   thumbnail_path = g_file_info_get_attribute_byte_string (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH);
 
   if (thumbnail_path)
     {
-      pixbuf = gdk_pixbuf_new_from_file_at_size (thumbnail_path,
-                                                 icon_size*scale, icon_size*scale,
-                                                 NULL);
+      GdkTexture *texture;
 
-      if (pixbuf != NULL)
-        return G_ICON (pixbuf);
+      texture = gdk_texture_new_from_filename_at_scale (thumbnail_path,
+                                                        icon_size*scale, icon_size*scale,
+                                                        NULL,
+                                                        NULL);
+
+      if (texture != NULL)
+        return G_ICON (texture);
     }
 
   icon = g_file_info_get_icon (info);
@@ -479,3 +484,12 @@ _gtk_file_info_get_icon (GFileInfo    *info,
   icon = g_themed_icon_new ("text-x-generic");
   return icon;
 }
+
+GFile *
+_gtk_file_info_get_file (GFileInfo *info)
+{
+  g_assert (G_IS_FILE_INFO (info));
+  g_assert (g_file_info_has_attribute (info, "standard::file"));
+
+  return G_FILE (g_file_info_get_attribute_object (info, "standard::file"));
+}
