@@ -36,6 +36,7 @@
 #include "gtkheaderbar.h"
 #include "gtkactionable.h"
 #include "gtkeventcontrollerkey.h"
+#include "gtkaccessible.h"
 
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
@@ -61,7 +62,10 @@ struct _GtkFontChooserDialogClass
  *
  * The `GtkFontChooserDialog` widget is a dialog for selecting a font.
  *
- * ![An example GtkFontChooserDialog](fontchooser.png)
+ * <picture>
+ *   <source srcset="fontchooser-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkFontChooserDialog" src="fontchooser.png">
+ * </picture>
  *
  * `GtkFontChooserDialog` implements the [iface@Gtk.FontChooser] interface
  * and does not provide much API of its own.
@@ -74,6 +78,11 @@ struct _GtkFontChooserDialogClass
  * interface exposes the buttons with the names “select_button”
  * and “cancel_button”.
  *
+ * ## CSS nodes
+ *
+ * `GtkFontChooserDialog` has a single CSS node with the name `window` and style
+ * class `.fontchooser`.
+ *
  * Deprecated: 4.10: Use [class@Gtk.FontDialog] instead
  */
 
@@ -184,6 +193,10 @@ setup_tweak_button (GtkFontChooserDialog *dialog)
       gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
       gtk_button_set_icon_name (GTK_BUTTON (button), "emblem-system-symbolic");
       gtk_widget_set_tooltip_text (button, _("Change Font Features"));
+      gtk_accessible_update_property (GTK_ACCESSIBLE (button),
+                                      GTK_ACCESSIBLE_PROPERTY_LABEL,
+                                      _("Change Font Features"),
+                                      -1);
 
       header = gtk_dialog_get_header_bar (GTK_DIALOG (dialog));
       gtk_header_bar_pack_end (GTK_HEADER_BAR (header), button);
