@@ -15,15 +15,17 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_FONT_CHOOSER_WIDGET_PRIVATE_H__
-#define __GTK_FONT_CHOOSER_WIDGET_PRIVATE_H__
+#pragma once
 
-#include "gtkfontchooserwidget.h"
+#include "deprecated/gtkfontchooserwidget.h"
+#include "gtkfilter.h"
 
 G_BEGIN_DECLS
 
 GAction *gtk_font_chooser_widget_get_tweak_action (GtkWidget *fontchooser);
 
+void     gtk_font_chooser_widget_set_filter (GtkFontChooserWidget *widget,
+                                             GtkFilter            *filter);
+
 G_END_DECLS
 
-#endif /* __GTK_FONT_CHOOSER_WIDGET_PRIVATE_H__ */
