@@ -38,9 +38,12 @@
 /**
  * GtkHeaderBar:
  *
- * A widget for creating custom title bars for windows.
+ * Creates a custom titlebar for a window.
  *
- * ![An example GtkHeaderBar](headerbar.png)
+ * <picture>
+ *   <source srcset="headerbar-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkHeaderBar" src="headerbar.png">
+ * </picture>
  *
  * `GtkHeaderBar` is similar to a horizontal `GtkCenterBox`. It allows
  * children to be placed at the start or the end. In addition, it allows
@@ -131,6 +134,7 @@ struct _GtkHeaderBar
   char *decoration_layout;
 
   guint show_title_buttons : 1;
+  guint use_native_controls : 1;
   guint track_default_decoration : 1;
 };
 
@@ -146,6 +150,7 @@ enum {
   PROP_TITLE_WIDGET,
   PROP_SHOW_TITLE_BUTTONS,
   PROP_DECORATION_LAYOUT,
+  PROP_USE_NATIVE_CONTROLS,
   LAST_PROP
 };
 
@@ -166,6 +171,9 @@ create_window_controls (GtkHeaderBar *bar)
   g_object_bind_property (bar, "decoration-layout",
                           controls, "decoration-layout",
                           G_BINDING_SYNC_CREATE);
+  g_object_bind_property (bar, "use-native-controls",
+                          controls, "use-native-controls",
+                          G_BINDING_SYNC_CREATE);
   g_object_bind_property (controls, "empty",
                           controls, "visible",
                           G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
@@ -176,6 +184,9 @@ create_window_controls (GtkHeaderBar *bar)
   g_object_bind_property (bar, "decoration-layout",
                           controls, "decoration-layout",
                           G_BINDING_SYNC_CREATE);
+  g_object_bind_property (bar, "use-native-controls",
+                          controls, "use-native-controls",
+                          G_BINDING_SYNC_CREATE);
   g_object_bind_property (controls, "empty",
                           controls, "visible",
                           G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
@@ -419,6 +430,10 @@ gtk_header_bar_get_property (GObject    *object,
       g_value_set_string (value, gtk_header_bar_get_decoration_layout (bar));
       break;
 
+    case PROP_USE_NATIVE_CONTROLS:
+      g_value_set_boolean (value, gtk_header_bar_get_use_native_controls (bar));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -447,6 +462,10 @@ gtk_header_bar_set_property (GObject      *object,
       gtk_header_bar_set_decoration_layout (bar, g_value_get_string (value));
       break;
 
+    case PROP_USE_NATIVE_CONTROLS:
+      gtk_header_bar_set_use_native_controls (bar, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -601,6 +620,25 @@ gtk_header_bar_class_init (GtkHeaderBarClass *class)
                            NULL,
                            GTK_PARAM_READWRITE);
 
+  /**
+   * GtkHeaderBar:use-native-controls:
+   *
+   * Whether to show platform native close/minimize/maximize buttons.
+   *
+   * For macOS, the [property@Gtk.HeaderBar:decoration-layout] property
+   * can be used to enable/disable controls.
+   *
+   * On Linux, this option has no effect.
+   *
+   * See also [Using GTK on Apple macOS](osx.html?native-window-controls).
+   *
+   * Since: 4.18
+   */
+  header_bar_props[PROP_USE_NATIVE_CONTROLS] =
+      g_param_spec_boolean ("use-native-controls", NULL, NULL,
+                            FALSE,
+                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, header_bar_props);
 
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
@@ -614,6 +652,7 @@ gtk_header_bar_init (GtkHeaderBar *bar)
   bar->title_widget = NULL;
   bar->decoration_layout = NULL;
   bar->show_title_buttons = TRUE;
+  bar->use_native_controls = FALSE;
 
   bar->handle = gtk_window_handle_new ();
   gtk_widget_set_parent (bar->handle, GTK_WIDGET (bar));
@@ -821,3 +860,50 @@ gtk_header_bar_get_decoration_layout (GtkHeaderBar *bar)
 
   return bar->decoration_layout;
 }
+
+/**
+ * gtk_header_bar_get_use_native_controls:
+ * @bar: a header bar
+ *
+ * Returns whether this header bar shows platform
+ * native window controls.
+ *
+ * Returns: true if native window controls are shown
+ *
+ * Since: 4.18
+ */
+gboolean
+gtk_header_bar_get_use_native_controls (GtkHeaderBar *bar)
+{
+  return bar->use_native_controls;
+}
+
+/**
+ * gtk_header_bar_set_use_native_controls:
+ * @bar: a header bar
+ * @setting: true to show native window controls
+ *
+ * Sets whether this header bar shows native window controls.
+ *
+ * This option shows the "stoplight" buttons on macOS.
+ * For Linux, this option has no effect.
+ *
+ * See also [Using GTK on Apple macOS](osx.html?native-window-controls).
+ *
+ * Since: 4.18
+ */
+void
+gtk_header_bar_set_use_native_controls (GtkHeaderBar *bar,
+                                         gboolean      setting)
+{
+  g_return_if_fail (GTK_IS_HEADER_BAR (bar));
+
+  setting = setting != FALSE;
+
+  if (bar->use_native_controls == setting)
+    return;
+
+  bar->use_native_controls = setting;
+
+  g_object_notify_by_pspec (G_OBJECT (bar), header_bar_props[PROP_USE_NATIVE_CONTROLS]);
+}
