@@ -26,5 +26,10 @@ G_BEGIN_DECLS
 int _gtk_label_get_cursor_position (GtkLabel *label);
 int _gtk_label_get_selection_bound (GtkLabel *label);
 
+void
+gtk_label_get_layout_location (GtkLabel  *self,
+                               float     *xp,
+                               float     *yp);
+
 G_END_DECLS
 
