@@ -18,8 +18,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_MENU_BUTTON_PRIVATE_H__
-#define __GTK_MENU_BUTTON_PRIVATE_H__
+#pragma once
 
 #include <gtk/gtkmenubutton.h>
 
@@ -27,4 +26,3 @@ G_BEGIN_DECLS
 
 G_END_DECLS
 
-#endif /* __GTK_MENU_BUTTON_PRIVATE_H__ */
