@@ -26,17 +26,22 @@
 #include "gtkprivate.h"
 #include "gtkwidgetprivate.h"
 #include "gtktypebuiltins.h"
-#include "gtkintl.h"
 
 
 /**
  * GtkOrientable:
  *
- * The `GtkOrientable` interface is implemented by all widgets that can be
- * oriented horizontally or vertically.
+ * An interface for widgets that can be oriented horizontally or vertically.
  *
  * `GtkOrientable` is more flexible in that it allows the orientation to be
  * changed at runtime, allowing the widgets to “flip”.
+ *
+ * ## CSS nodes
+ *
+ * `GtkWidget` types implementing the `GtkOrientable` interface will
+ * automatically acquire the `horizontal` or `vertical` CSS class depending on
+ * the value of the [property@Gtk.Orientable:orientation] property.
+
  */
 
 
@@ -47,7 +52,7 @@ static void
 gtk_orientable_default_init (GtkOrientableInterface *iface)
 {
   /**
-   * GtkOrientable:orientation: (attributes org.gtk.Property.get=gtk_orientable_get_orientation org.gtk.Property.set=gtk_orientable_set_orientation)
+   * GtkOrientable:orientation:
    *
    * The orientation of the orientable.
    **/
@@ -59,7 +64,7 @@ gtk_orientable_default_init (GtkOrientableInterface *iface)
 }
 
 /**
- * gtk_orientable_set_orientation: (attributes org.gtk.Method.set_property=orientation)
+ * gtk_orientable_set_orientation:
  * @orientable: a `GtkOrientable`
  * @orientation: the orientable’s new orientation
  *
@@ -80,7 +85,7 @@ gtk_orientable_set_orientation (GtkOrientable  *orientable,
 }
 
 /**
- * gtk_orientable_get_orientation: (attributes org.gtk.Method.get_property=orientation)
+ * gtk_orientable_get_orientation:
  * @orientable: a `GtkOrientable`
  *
  * Retrieves the orientation of the @orientable.
