@@ -77,6 +77,7 @@ G_DEFINE_BOXED_TYPE (GtkPrintSetup, gtk_print_setup,
                      gtk_print_setup_ref,
                      gtk_print_setup_unref)
 
+#ifdef HAVE_GIO_UNIX
 static GtkPrintSetup *
 gtk_print_setup_new (void)
 {
@@ -88,6 +89,7 @@ gtk_print_setup_new (void)
 
   return setup;
 }
+#endif
 
 /**
  * gtk_print_setup_ref:
@@ -151,12 +153,14 @@ gtk_print_setup_get_print_settings (GtkPrintSetup *setup)
   return setup->print_settings;
 }
 
+#ifdef HAVE_GIO_UNIX
 static void
 gtk_print_setup_set_print_settings (GtkPrintSetup    *setup,
                                     GtkPrintSettings *print_settings)
 {
   g_set_object (&setup->print_settings, print_settings);
 }
+#endif
 
 /**
  * gtk_print_setup_get_page_setup:
@@ -177,6 +181,7 @@ gtk_print_setup_get_page_setup (GtkPrintSetup *setup)
   return setup->page_setup;
 }
 
+#ifdef HAVE_GIO_UNIX
 static void
 gtk_print_setup_set_page_setup (GtkPrintSetup *setup,
                                 GtkPageSetup  *page_setup)
@@ -187,7 +192,6 @@ gtk_print_setup_set_page_setup (GtkPrintSetup *setup,
 static GtkPrinter *
 gtk_print_setup_get_printer (GtkPrintSetup *setup)
 {
-#ifdef HAVE_GIO_UNIX
   if (!setup->printer)
     {
       const char *name = NULL;
@@ -198,7 +202,6 @@ gtk_print_setup_get_printer (GtkPrintSetup *setup)
       if (name)
         setup->printer = gtk_printer_find (name);
     }
-#endif
 
   return setup->printer;
 }
@@ -209,6 +212,7 @@ gtk_print_setup_set_printer (GtkPrintSetup *setup,
 {
   g_set_object (&setup->printer, printer);
 }
+#endif
 
 /* }}} */
 /* {{{ GObject implementation */
@@ -937,6 +941,8 @@ prepare_print_response (GDBusConnection *connection,
         setup->token = token;
 
         g_task_return_pointer (task, gtk_print_setup_ref (setup), (GDestroyNotify) gtk_print_setup_unref);
+
+        gtk_print_setup_unref (setup);
       }
       break;
 
@@ -1539,8 +1545,10 @@ gtk_print_dialog_setup (GtkPrintDialog       *self,
  * which contains the print settings and page setup information that
  * will be used to print.
  *
- * Returns: The `GtkPrintSetup` object that resulted from the call,
- *   or `NULL` if the call was not successful
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
+ * Returns: (transfer full): the resulting `[struct@Gtk.PrintSetup]`
  *
  * Since: 4.14
  */
@@ -1679,6 +1687,9 @@ gtk_print_dialog_print (GtkPrintDialog       *self,
  * call may not be instant as it operation will for the printer to finish
  * printing.
  *
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
  * Returns: (transfer full): a [class@Gio.OutputStream]
  *
  * Since: 4.14
@@ -1817,6 +1828,9 @@ gtk_print_dialog_print_file (GtkPrintDialog       *self,
  * Finishes the [method@Gtk.PrintDialog.print_file] call and
  * returns the results.
  *
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
  * Returns: Whether the call was successful
  *
  * Since: 4.14
