@@ -17,8 +17,7 @@
  * Authors: Benjamin Otte <otte@gnome.org>
  */
 
-#ifndef __GTK_SCALER_H__
-#define __GTK_SCALER_H__
+#pragma once
 
 #include <gdk/gdk.h>
 
@@ -29,8 +28,7 @@ G_BEGIN_DECLS
 G_DECLARE_FINAL_TYPE (GtkScaler, gtk_scaler, GTK, SCALER, GObject)
 
 GdkPaintable *  gtk_scaler_new                  (GdkPaintable   *paintable,
-                                                 double          scale_factor);
+                                                 double          scale);
 
 G_END_DECLS
 
-#endif /* __GTK_SCALER_H__ */
