@@ -15,8 +15,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _GTK_INSPECTOR_CLIPBOARD_H_
-#define _GTK_INSPECTOR_CLIPBOARD_H_
+#pragma once
 
 #include <gtk/gtkwidget.h>
 
@@ -35,4 +34,3 @@ void            gtk_inspector_clipboard_set_display             (GtkInspectorCli
 
 G_END_DECLS
 
-#endif // _GTK_INSPECTOR_CLIPBOARD_H_
