@@ -1,7 +1,6 @@
 
 
-#ifndef __GTK_LAYOUT_OVERLAY_H__
-#define __GTK_LAYOUT_OVERLAY_H__
+#pragma once
 
 #include "inspectoroverlay.h"
 
@@ -16,4 +15,3 @@ G_END_DECLS
 
 
 
-#endif
