@@ -41,6 +41,7 @@ typedef struct _GtkInspectorRenderRecording
   GdkRectangle area;
   cairo_region_t *clip_region;
   GskRenderNode *node;
+  gpointer surface;
 } GtkInspectorRenderRecording;
 
 typedef struct _GtkInspectorRenderRecordingClass
@@ -54,7 +55,8 @@ GtkInspectorRecording *
                 gtk_inspector_render_recording_new           (gint64                             timestamp,
                                                               const GdkRectangle                *area,
                                                               const cairo_region_t              *clip_region,
-                                                              GskRenderNode                     *node);
+                                                              GskRenderNode                     *node,
+                                                              gpointer                           surface);
 
 GskRenderNode * gtk_inspector_render_recording_get_node      (GtkInspectorRenderRecording       *recording);
 const cairo_region_t *
@@ -62,6 +64,9 @@ const cairo_region_t *
 const cairo_rectangle_int_t *
                 gtk_inspector_render_recording_get_area      (GtkInspectorRenderRecording       *recording);
 
+gpointer
+                gtk_inspector_render_recording_get_surface   (GtkInspectorRenderRecording       *recording);
+
 G_END_DECLS
 
 
