@@ -25,12 +25,12 @@
 #include <gtk/gtkmodulesprivate.h>
 #include "gtkprintbackendcpdb.h"
 #include "gtkprintbackendutils.h"
+#include "gtkprintercpdb.h"
 
 #include <cairo.h>
 #include <cairo-pdf.h>
 
 #include <glib/gi18n-lib.h>
-#include <gtkprintercpdb.h>
 
 #define GTK_PRINT_BACKEND_CPDB_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_PRINT_BACKEND_CPDB, GtkPrintBackendCpdbClass))
 #define GTK_IS_PRINT_BACKEND_CPDB_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_PRINT_BACKEND_CPDB))
