@@ -604,32 +604,6 @@ run_tests (GtkTextBuffer *buffer)
   check_specific_tag (buffer, "end_tag");
 }
 
-
-static const char  *book_closed_xpm[] = {
-"16 16 6 1",
-"       c None s None",
-".      c black",
-"X      c red",
-"o      c yellow",
-"O      c #808080",
-"#      c white",
-"                ",
-"       ..       ",
-"     ..XX.      ",
-"   ..XXXXX.     ",
-" ..XXXXXXXX.    ",
-".ooXXXXXXXXX.   ",
-"..ooXXXXXXXXX.  ",
-".X.ooXXXXXXXXX. ",
-".XX.ooXXXXXX..  ",
-" .XX.ooXXX..#O  ",
-"  .XX.oo..##OO. ",
-"   .XX..##OO..  ",
-"    .X.#OO..    ",
-"     ..O..      ",
-"      ..        ",
-"                "};
-
 static void
 fill_buffer (GtkTextBuffer *buffer)
 {
@@ -638,7 +612,6 @@ fill_buffer (GtkTextBuffer *buffer)
   GdkRGBA color2;
   GtkTextIter iter;
   GtkTextIter iter2;
-  GdkPixbuf *pixbuf;
   GdkTexture *texture;
   int i;
 
@@ -676,8 +649,7 @@ fill_buffer (GtkTextBuffer *buffer)
                               "font", "-*-courier-bold-r-*-*-10-*-*-*-*-*-*-*",
                               NULL);
 
-  pixbuf = gdk_pixbuf_new_from_xpm_data (book_closed_xpm);
-  texture = gdk_texture_new_for_pixbuf (pixbuf);
+  texture = gdk_texture_new_from_resource ("/org/gtk/libgtk/icons/16x16/actions/audio-volume-high.png");
 
   g_assert_nonnull (texture);
 
@@ -789,7 +761,6 @@ fill_buffer (GtkTextBuffer *buffer)
 
   gtk_text_buffer_apply_tag (buffer, tag, &iter, &iter2);
 
-  g_object_unref (pixbuf);
   g_object_unref (texture);
 }
 
@@ -1953,11 +1924,11 @@ test_serialize_wrap_mode (void)
 int
 main (int argc, char** argv)
 {
-  /* First, we turn on btree debugging. */
-  gtk_set_debug_flags (gtk_get_debug_flags () | GTK_DEBUG_TEXT);
-
   gtk_test_init (&argc, &argv);
 
+  /* Turn on btree debugging */
+  gtk_set_debug_flags (gtk_get_debug_flags () | GTK_DEBUG_TEXT);
+
   g_test_add_func ("/TextBuffer/UTF8 unknown char", test_utf8);
   g_test_add_func ("/TextBuffer/Line separator", test_line_separator);
   g_test_add_func ("/TextBuffer/Backspace", test_backspace);
